;****************************************************************************
; Program	: Port Output 02
; Description	: LED Running with sound on Piezo
; For		: MCS-51 Microcontroller Training System
; Filename	: lab1102.asm
; Assembler	: RAD51
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DRIVER_LE	BIT	P1.4	; Driver Latch Enable (Active High : Level)
PIEZO		BIT	P3.4	; PIEZO driver (Active High : Level)

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		MOV	P0,#00000000B	; Clear Databus

MAIN:		MOV	A,#01010101B	; Initial ACC. Value

LOOP_LED:	MOV	P0,A		; Out ACC. Value to DATABUS
		ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock
		MOV	R4,#200		; Set 200 times loop

LOOP_SOUND:	CPL	PIEZO		; Complement PIEZO
		ACALL	DELAY_100us	; Delay 100us
		DJNZ	R4,LOOP_SOUND	; Do loop until 200 times

		CPL	A		; Complement ACC.
		ACALL	DELAY_1s	; Delay 1 s
		AJMP	LOOP_LED	; Jump to loop LED

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
DRIVE_LE_CLK:	SETB	DRIVER_LE	; Driver Latch Enable
		NOP
		NOP
		CLR	DRIVER_LE	; Driver Latch Disable
		NOP
		RET

;----------------------------------------------------------------------------
; Dummy Delay time 100us,10m,1s
;----------------------------------------------------------------------------
DELAY_100us:	MOV	R6,#017H	; Each loop = 100 us
DELAY_100us_1:	NOP
		NOP
		DJNZ	R6,DELAY_100us_1
		RET

DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_1s:	MOV	R5,#100		; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

		END