;****************************************************************************
; Program	: Port Input 02
; Description	: Key Input from S1(P3.2) Shift LED Left /S2(P3.3) Shift Right
; For		: MCS-51 Microcontroller Training System
; Filename	: lab1202.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DRIVER_LE	BIT	P1.4	; Driver Latch Enable (Active High : Level)

LEFT_SHF	BIT	P3.2	; Interrupt 0 PIN S2
RIGHT_SHF	BIT	P3.3	; Interrupt 1 PIN S1

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		MOV	P0,#00000000B	; Clear Databus
		CLR	DRIVER_LE	; Clear latch enable

MAIN:		MOV	A,#10000000B	; Initial ACC. Value
		MOV	P0,A		; Out ACC. Value to DATABUS
		ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock

LOOP:		JB	LEFT_SHF,CHK_OFF ; Check S2 pressed?
		ACALL	DELAY_10ms	; Delay to avoid switch bounce
		JB	LEFT_SHF,CHK_OFF ; Repeat check S2 Pressed?
		RL	A
		MOV	P0,A		; Out ACC. Value to DATABUS
		ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock
		ACALL	DELAY_200ms	; Delay 200ms
		AJMP	LOOP		; Jump to Loop
				
CHK_OFF:	JB	RIGHT_SHF,LOOP	; Check S1 pressed?
		ACALL	DELAY_10ms	; Delay to avoid switch bounce
		JB	RIGHT_SHF,LOOP	; Repeat check S1 Pressed?
		RR	A
		MOV	P0,A		; Out ACC. Value to DATABUS
		ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock
		ACALL	DELAY_200ms	; Delay 200ms
		AJMP	LOOP		; Jump to Loop

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
DRIVE_LE_CLK:	SETB	DRIVER_LE	; Driver Latch Enable
		NOP			; NOP
		CLR	DRIVER_LE	; Driver Latch Disable
		NOP
		RET

;----------------------------------------------------------------------------
; Dummy Delay time 10m,100ms
;----------------------------------------------------------------------------
DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_200ms:	MOV	R7,#200		; Do 200 times
DELAY_200ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_200ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_200ms_2
		DJNZ	R7,DELAY_200ms_1
		RET

		END