;****************************************************************************
; Program	: Keypad 01
; Description	: Keypad input and show key value to DSP with shift display
; For		: MCS-51 Microcontroller Training System
; Filename	: lab1401.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DSP1		BIT	P1.2	; 7 Segment DSP1 enable (Active Low : Level)
DSP2		BIT	P1.3	; 7 Segment DSP2 enable (Active Low : Level)

KPAD_ROW0	BIT	P2.0	; Keypad Input Row 0
KPAD_ROW1	BIT	P2.1	; Keypad Input Row 1
KPAD_ROW2	BIT	P2.2	; Keypad Input Row 2
KPAD_ROW3	BIT	P2.3	; Keypad Input Row 3
KPAD_COL2	BIT	P2.4	; Keypad Output Column 2
KPAD_COL1	BIT	P2.5	; Keypad Output Column 1
KPAD_COL0	BIT	P2.6	; Keypad Output Column 0

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG		EQU	02FH	; User FLAG
KEYPRESSED	BIT	FLAG.0	; Define KEYPRESS as bit
R_SHF		BIT	FLAG.1	; Define Right Shift as bit

DSP1_BUFFER	EQU	030H	; For keep DSP1 Data
DSP2_BUFFER	EQU	031H	; For keep DSP2 Data
KPAD_DATA	EQU	032H	; For keep Keypad Data

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		MOV	P0,#00000000B	; Clear Databus
		SETB	DSP1		; Clear DSP1
		SETB	DSP2		; Clear DSP2
		MOV	P2,#11111111B	; Clear status keypad and 1-Wire

MAIN:		MOV	DSP1_BUFFER,#0	; Clear DSP1 Buffer
		MOV	DSP2_BUFFER,#0	; Clear DSP2 Buffer
		MOV	FLAG,#0		; Clear FLAG Status

LOOP:		ACALL	GET_KPAD	; Get Keypad Data
		MOV	A,KPAD_DATA
		JZ	NEXT		; Check Keypad Pressed?
		JB	KEYPRESSED,SHOW ; Check Keypad still pressed?
		SETB	KEYPRESSED	; Set bit keypressed

LEFT_SHIFT:	MOV	DSP2_BUFFER,DSP1_BUFFER ; Shift Display to Right
		MOV	DSP1_BUFFER,KPAD_DATA	; Shift Keypad data in
		AJMP	SHOW			; Jump to Show Display

NEXT:		CLR	KEYPRESSED	; Clear bit keypressed
SHOW:		ACALL	SHOW_DSP	; Call Show DSP Subroutine
		CLR	R_SHF		; Clear bit Right Shift
		AJMP	LOOP		; Jump to loop

;----------------------------------------------------------------------------
; Keypad Scan key Subroutine
;----------------------------------------------------------------------------
GET_KPAD:	MOV	P2,#0FFH	; Pull P2 to High
		MOV	KPAD_DATA,#0	; Clear Keypad Data

CHK_COL0:	CLR	KPAD_COL0	; Begin Scan Column 0
		MOV	A,P2		; Get Port2 Value
		ANL	A,#00FH		; Get only lower 4 bit
		CJNE	A,#00FH,COL0_DETECT ; Check All rows '1'?
		AJMP	CHK_COL1	; All rows '1' => check next column

COL0_DETECT:	MOV	KPAD_DATA,#01	; Initial KPAD_DATA = 1
		AJMP	GET_ROW		; Jump to get row value

CHK_COL1:	SETB	KPAD_COL0	; Stop Scan Column 0
		CLR	KPAD_COL1	; Begin Scan Column 1
		MOV	A,P2		; Get Port2 Value
		ANL	A,#00FH		; Get only lower 4 bit
		CJNE	A,#00FH,COL1_DETECT ; Check All rows '1'?
		AJMP	CHK_COL2	; All rows '1' => check next column

COL1_DETECT:	MOV	KPAD_DATA,#02	; Initial KPAD_DATA = 2
		AJMP	GET_ROW		; Jump to get row value

CHK_COL2:	SETB	KPAD_COL1	; Stop Scan Column 1
		CLR	KPAD_COL2	; Begin Scan Column 2
		MOV	A,P2		; Get Port2 Value
		ANL	A,#00FH		; Get only lower 4 bit
		CJNE	A,#00FH,COL2_DETECT ; Check All rows '1'?
		RET			; All rows '1' => return

COL2_DETECT:	MOV	KPAD_DATA,#03	; Initial KPAD_DATA = 3

GET_ROW:	CLR	KPAD_COL0	; Enable all Column to find Crosspoint
		CLR	KPAD_COL1
		CLR	KPAD_COL2

		JB	KPAD_ROW0,CHK_ROW1 ; Check Row 0 Detect?
		RET			; Row 0 Detect => return

CHK_ROW1:	JB	KPAD_ROW1,CHK_ROW2 ; Check Row 2 Detect?
		MOV	A,KPAD_DATA	; Add 3 with KPAD_DATA
		ADD	A,#3
		MOV	KPAD_DATA,A
		RET			; Return

CHK_ROW2:	JB	KPAD_ROW2,CHK_ROW3 ; Check Row 2 Detect?
		MOV	A,KPAD_DATA		; Add 6 with KPAD_DATA
		ADD	A,#6
		MOV	KPAD_DATA,A
		RET			; Return

CHK_ROW3:	MOV	A,KPAD_DATA		; Add 9 with KPAD_DATA
		ADD	A,#9
		MOV	KPAD_DATA,A
		RET			; Return

;----------------------------------------------------------------------------
; Show DSP Subroutine
;----------------------------------------------------------------------------
SHOW_DSP:	MOV	R4,#5		; Set loop 5 times
SCAN_DSP_LOOP:	MOV	A,DSP1_BUFFER	; Restore DSP1 to Display
		MOV	DPTR,#DSP_BLANK ; Move DIGIT Start Pointer
		MOVC	A,@A+DPTR	; Get ROM Data from Pointer+ACC.
		MOV	P0,A		; Out ACC. to DATABUS
		CLR	DSP1		; Enable DSP1
		ACALL	DELAY_1ms	; Delay
		SETB	DSP1		; Disable DSP1

		MOV	A,DSP2_BUFFER	; Restore DSP2 to Display
		MOV	DPTR,#DSP_BLANK ; Move DIGIT Start Pointer
		MOVC	A,@A+DPTR	; Get ROM Data from Pointer+ACC.
		MOV	P0,A		; Out ACC. to DATABUS
		CLR	DSP2		; Enable DSP2
		ACALL	DELAY_1ms	; Delay
		SETB	DSP2		; Disable DSP1
		DJNZ	R4,SCAN_DSP_LOOP; Do until 5 times
		RET

;----------------------------------------------------------------------------
; Dummy Delay time 1m,10ms,1s
;----------------------------------------------------------------------------
DELAY_1ms:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_1ms_1:	NOP
		NOP
		DJNZ	R6,DELAY_1ms_1
		RET

DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_1s:	MOV	R5,#100		; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;		Segment	.GFEDCBA
DSP_BLANK:	DB	10000000B
DSP_NUM1:	DB	00000110B
DSP_NUM2:	DB	01011011B
DSP_NUM3:	DB	01001111B
DSP_NUM4:	DB	01100110B
DSP_NUM5:	DB	01101101B
DSP_NUM6:	DB	01111101B
DSP_NUM7:	DB	00000111B
DSP_NUM8:	DB	01111111B
DSP_NUM9:	DB	01101111B
DSP_STAR:	DB	01110110B
DSP_NUM0:	DB	00111111B
DSP_HASH:	DB	01100011B

		END