;****************************************************************************
; Program	: LCD Display 02
; Description	: LCD Display 16x2 Show , 16 Char from ROM , Shift Display
;		: 4 bit Mode Data
; For		: MCS-51 Microcontroller Training System
; Filename	: P1502.ASM
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
LCD_EN		BIT	P3.6	; LCD Module Enable (Active High : Level)
LCD_RS		BIT	P3.7	; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
LCD_ADDR	EQU	030H	; For keep LCD Address
LCD_DATA	EQU	031H	; For keep LCD Data

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		MOV	P0,#00000000B	; Clear Databus
		CLR	LCD_EN		; Clear LCD Enable
		CLR	LCD_RS		; Clear LCD RS

MAIN:		ACALL	INIT_LCD	; Call LCD Initial subroutine

LOOP:		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-0FH (Increase automatic)
		MOV	DPTR,#TITLE_3	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 10H-17H (Increase automatic)

		MOV	LCD_ADDR,#040H	; Set Address 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE_2	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 40H-4FH (Increase automatic)
		MOV	DPTR,#TITLE_4	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 50H-5FH (Increase automatic)

		ACALL	DELAY_1s	; Delay
		ACALL	DELAY_1s

		MOV	R4,#16		; Set Loop 16 times
LOOP_LCD_L_SHF: ACALL	LCD_LSHF	; Left Shift LCD Display
		ACALL	DELAY_100ms
		ACALL	DELAY_100ms
		DJNZ	R4,LOOP_LCD_L_SHF ; Do until 16 times

		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE_5	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-0FH (Increase automatic)

		MOV	LCD_ADDR,#040H	; Set Address 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE_6	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 40H-4FH (Increase automatic)

		ACALL	DELAY_1s	; Delay
		ACALL	DELAY_1s

		MOV	R4,#16		; Set Loop 16 times
LOOP_LCD_R_SHF: ACALL	LCD_RSHF	; Right Shift LCD Display
		ACALL	DELAY_100ms	; Delay
		ACALL	DELAY_100ms
		DJNZ	R4,LOOP_LCD_R_SHF	; Do until 16 times

		ACALL	DELAY_1s	; Delay
		ACALL	DELAY_1s

		ACALL	LCD_CLR		; Clear LCD Display

		MOV	LCD_ADDR,#002H	; Set Address 02H
		ACALL	SET_ADDR_LCD
		ACALL	LCD_BLINK	; Blink Cursor
		ACALL	DELAY_1s

		MOV	LCD_DATA,#'H'	; Write Character 'H'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s

		MOV	LCD_DATA,#'e'	; Write Character 'e'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s	; Delay

		MOV	LCD_DATA,#'l'	; Write Character 'l'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s	; Delay

		MOV	LCD_DATA,#'l'	; Write Character 'l'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s	; Delay

		MOV	LCD_DATA,#'o'	; Write Character 'o'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s	; Delay

		MOV	LCD_DATA,#' '	; Write Space
		ACALL	WRCHAR_LCD
		ACALL	WRCHAR_LCD
		MOV	LCD_DATA,#'W'	; Write Character 'W'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s	; Delay

		MOV	LCD_DATA,#'o'	; Write Character 'o'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s	; Delay

		MOV	LCD_DATA,#'r'	; Write Character 'r'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s	; Delay

		MOV	LCD_DATA,#'l'	; Write Character 'l'
		ACALL	WRCHAR_LCD
		ACALL	DELAY_1s	; Delay

		MOV	LCD_DATA,#'d'	; Write Character 'd'
		ACALL	WRCHAR_LCD

		MOV	LCD_ADDR,#040H	; Set Address 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#STRING_1	; Index Pointer ROM to Show LCD
		ACALL	STRING_LCD	; 40H-4FH (Increase automatic)

		ACALL	DELAY_1s	; Delay

		ACALL	LCD_OFF		; Off Display
		ACALL	DELAY_1s	; Delay

		ACALL	LCD_ON		; On Display
		ACALL	DELAY_1s	; Delay

		ACALL	LCD_OFF		; Off Display
		ACALL	DELAY_1s	; Delay

		ACALL	LCD_ON		; On Display
		ACALL	DELAY_1s	; Delay
		ACALL	DELAY_1s

		ACALL	LCD_CLR

		AJMP	LOOP		; Jump to loop


;----------------------------------------------------------------------------
; Define LCD's command constant
;----------------------------------------------------------------------------
LCD_CMD_CLR	EQU	00000001B	; Clear
LCD_CMD_HOME	EQU	00000010B	; Cursor home
LCD_CMD_AUTOINC	EQU	00000110B	; Auto increment
LCD_CMD_OFF	EQU	00001000B	; Off
LCD_CMD_ON	EQU	00001100B	; On
LCD_CMD_BLINK	EQU	00001111B	; Cursor on & blink
LCD_CMD_LSHF	EQU	00011000B	; Shift display to left
LCD_CMD_RSHF	EQU	00011100B	; Shift display to right


;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:	ACALL   DELAY_100ms	; Delay

		CLR	LCD_RS		; Clear LCD_RS pin
		ANL	P0,#00FH	; Force 8 bit command first times
		ORL	P0,#00110000B	; by modified to P0.4-0.7 only
		ACALL	LCD_CLK		; Pulse LCD clock
		ACALL	DELAY_10ms	; Delay

	; force previous possible mode (4 or 8 bit) to be 8 bit
		ACALL	LCD_CLK		; Pulse LCD clock

	; force back to 4 bit mode this times
		ANL	P0,#11101111B	; Force DL bit to 0 as 4 bit mode
		ACALL	LCD_CLK		; Pulse LCD clock

					; Re-setting up register to 4 bit (N=0 AND F=0 sent later)
		ANL	P0,#00FH	; Send upper 4 bit first
		ORL	P0,#00100000B	;
		ACALL	LCD_CLK		; Pulse LCD clock

		ANL	P0,#00FH	; Continue setting up register N=1 AND F=0 
		ORL	P0,#10000000B	; Send lower 4 bit
		ACALL	LCD_CLK		; Pulse LCD clock

		ACALL	LCD_OFF		; Display off
		ACALL	LCD_AUTOINC	; Set auto increment cursor after write
		ACALL	LCD_HOME	; Return cursor to home
		ACALL	LCD_CLR		; Display cleared
		ACALL	LCD_ON		; Display on
		RET

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:	MOV	LCD_DATA,#LCD_CMD_CLR	; Display cleared
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:	MOV	LCD_DATA,#LCD_CMD_HOME	; Return cursor to uppper left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_AUTOINC:	MOV	LCD_DATA,#LCD_CMD_AUTOINC; Cursor auto increment after write
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:	MOV	LCD_DATA,#LCD_CMD_OFF	; Display off
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:		MOV	LCD_DATA,#LCD_CMD_ON	; Display on
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:	MOV	LCD_DATA,#LCD_CMD_BLINK	; Display blink
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Left Shift Display
;----------------------------------------------------------------------------
LCD_LSHF:	MOV	LCD_DATA,#LCD_CMD_LSHF	; Shift display to left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Right Shift Display
;----------------------------------------------------------------------------
LCD_RSHF:	MOV	LCD_DATA,#LCD_CMD_RSHF	; Shift display to right
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:          LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR     LCD_RS		; Clear LCD_RS pin
		PUSH	ACC
		MOV	A,LCD_ADDR
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		SETB	P0.7		; Set MSB of data as DDRAM point cmd.
		ACALL   LCD_CLK		; Pulse LCD clock

		MOV	A,LCD_ADDR
		SWAP	A
		ANL	A,#0F0H
		ANL	P0,#00FH
		
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock
		POP	ACC
		RET			; Return

;----------------------------------------------------------------------------
; LCD Clear Display
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCMD_LCD:	CLR	LCD_RS		; Clear LCD_RS Pin

		PUSH	ACC
		MOV	A,LCD_DATA
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock

		MOV	A,LCD_DATA
		SWAP	A
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock
		POP	ACC

		RET			; Return

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:	SETB	LCD_RS		; Set LCD_RS Pin

		PUSH	ACC
		MOV	A,LCD_DATA
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock

		MOV	A,LCD_DATA
		SWAP	A
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock
		POP	ACC

		RET			; Return

;----------------------------------------------------------------------------
; Put string from ROM (string must ended with 000H)
; I/P:          DPTR : Locate ROM Address
;----------------------------------------------------------------------------
STRING_LCD:	PUSH	ACC		; Put ACC. into stack
STRING_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		CJNE	A,#0,STRING_LCD_2	; Do until null terminate detect
		POP	ACC		; Pop ACC from stack
		RET			; Return
				
STRING_LCD_2:
		MOV	LCD_DATA,A

		PUSH	ACC
		MOV	A,LCD_DATA
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock

		MOV	A,LCD_DATA
		SWAP	A
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock
		POP	ACC

		INC	DPTR		; Increase Pointer
		AJMP	STRING_LCD_1

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:	MOV	R0,#0		; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		MOV	LCD_DATA,A	; Move ACC. to LCD_DATA

		PUSH	ACC
		MOV	A,LCD_DATA
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock

		MOV	A,LCD_DATA
		SWAP	A
		ANL	A,#0F0H
		ANL	P0,#00FH
		ORL	P0,A
		ACALL   LCD_CLK		; Pulse LCD clock
		POP	ACC

		INC	DPTR		; Increase Pointer
		INC	R0		; Increase loop counter
		CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
		RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:	SETB	LCD_EN		; Pulse Clock to LCD_EN
		ACALL	LCD_DELAY
		CLR	LCD_EN
		ACALL	LCD_DELAY
		RET

;----------------------------------------------------------------------------
; Dummy Delay time LCD_DELAY, 10m, 100m, 1s
;----------------------------------------------------------------------------
LCD_DELAY:	MOV	R7,#002		; Do 2 times
LCD_DELAY_1:	MOV	R6,#0E6H	; Each loop = 1 ms
LCD_DELAY_2:	NOP
		NOP
		DJNZ	R6,LCD_DELAY_2
		DJNZ	R7,LCD_DELAY_1
		RET

DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_100ms:	MOV	R7,#100		; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

DELAY_1s:	MOV	R5,#100		; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;			 0123456789ABCDEF
TITLE_1:	DB	'LCD Module Show '
TITLE_2:	DB	'Using 4 bit mode'

TITLE_3:	DB	' Shift Display  '
TITLE_4:	DB	' Right and Left '

TITLE_5:	DB	' Write Character'
TITLE_6:	DB	' Cursor & Blink '

NULL		EQU	0
STRING_1:	DB	'LCD Module  16x2',NULL

		END