;****************************************************************************
; Program	: Stepper Motor 03
; Description	: Stepper Motor Half-step Drive
; 		  S1 rotate Left , S2 rotate Right
; For		: MCS-51 Microcontroller Training System
; Filename	: lab1603.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
LEFT_SW		BIT	P3.2	; Interrupt 0 PIN S2
RIGHT_SW	BIT	P3.3	; Interrupt 1 PIN S1

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
STEP		EQU	030H	; For keep Stepper State

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		MOV	P0,#00000000B	; Clear Databus

MAIN:		MOV	STEP,#0		; Initial Stepper

		MOV	DPTR,#TABLE_STEP; Point on table

LOOP:		MOV	A,STEP
		MOVC	A,@A+DPTR	; Get table
		MOV	B,P0
		ANL	B,#0F0H		; Save P0 higher 4 bit
		ORL	A,B
		MOV	P0,A		; Move Stepper Value to DATABUS

WAIT_SW:	JNB	LEFT_SW,ROTATE_LEFT ; Check S2 Pressed?
		JB	RIGHT_SW,WAIT_SW; Check S1 Pressed?
		AJMP	ROTATE_RIGHT	; S1 Pressed => Rotate Right

ROTATE_LEFT:	INC	STEP		; Increase STEP
		ANL	STEP,#00000111B	; Limit within 0-7 range
		ACALL	DELAY_100ms	; Delay
		AJMP	LOOP

ROTATE_RIGHT:	DEC	STEP		; Decrease STEP
		ANL	STEP,#00000111B	; Limit within 0-7 range
		ACALL	DELAY_100ms	; Delay
		AJMP	LOOP

;----------------------------------------------------------------------------
; Dummy Delay time 100m
;----------------------------------------------------------------------------
DELAY_100ms:	MOV	R7,#100		; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;		bit	3210	(ignore 4 bit higher, signed as 0)
TABLE_STEP:	DB	0001B
		DB	0011B
		DB	0010B
		DB	0110B
		DB	0100B
		DB	1100B
		DB	1000B
		DB	1001B

		END