;****************************************************************************
; Program	: R-2R Ladder D/A 01
; Description	: Staircase Signal Generator
; For		: MCS-51 Microcontroller Training System
; Filename	: lab1701.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
R2R_LE		BIT	P3.5	; R-2R D/A Latch Enable (Active High : Level)

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
R2R_DATA	EQU	030H	; For keep R-2R D/A Data

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		MOV	P0,#00000000B	; Clear Databus
		MOV	P3,#00011111B	; Clear status R-2R, P3.2-3.4

MAIN:		MOV	R2R_DATA,#0	; Clear R-2R D/A Data
LOOP:		MOV	P0,R2R_DATA	; Move R2R Data to DATABUS
		ACALL	R2R_LE_CLK	; Latch
		INC	R2R_DATA	; Increase R-2R DATA
					; (loop 0 -> FFH -> 0)

		ACALL	DELAY_100ms	; Delay
		AJMP	LOOP		; Jump to loop

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
R2R_LE_CLK:	SETB	R2R_LE		; R-2R Latch Enable
		NOP
		CLR	R2R_LE		; R-2R Latch Disable
		NOP
		RET

;----------------------------------------------------------------------------
; Dummy Delay time 1ms
;----------------------------------------------------------------------------
DELAY_1ms:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_1ms_1:	NOP
		NOP
		DJNZ	R6,DELAY_1ms_1
		RET

DELAY_100ms:	MOV	R7,#100		; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

		END