;****************************************************************************
; Program	: I2C PCF8591 A/D D/A 01
; Description	: I2C PCF8591 8 bit 4 Ch. A/D , 1 Ch. D/A, Sub address 000
; For		: MCS-51 Microcontroller Training System
; Filename	: lab1901.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;---------------------------------------------------------------------------;
; Define Port&Pin Name
;---------------------------------------------------------------------------;
SDA		BIT	P1.0	; SDA I2C Bus
SCL		BIT	P1.1	; SCL I2C Bus

LCD_EN		BIT	P3.6	; LCD Module Enable (Active High : Level)
LCD_RS		BIT	P3.7	; LCD Module Register Select

;---------------------------------------------------------------------------;
; Define User Register
;---------------------------------------------------------------------------;
FLAG		EQU	02FH	; User FLAG
I2C_ACK		BIT	FLAG.0	; Define I2C Acknowledge as bit

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
LCD_ADDR	EQU	030H	; For keep LCD Address
LCD_DATA	EQU	031H	; For keep LCD Data
LCD_PTR		EQU	032H	; For keep LCD 3 Cha.r Pointer
I2C_ADDR	EQU	033H	; For keep I2C Address
I2C_DATA	EQU	034H	; For keep I2C Data
CONTROL		EQU	035H	; For keep I2C PCF8591 Control Byte
DA_DATA		EQU	036H	; For keep I2C D/A Data
CHANNEL		EQU	037H	; For keep Channel Number
AD_DATA		EQU	038H	; For keep I2C A/D Data
BUFFER		EQU	039H	; Buffer
				; 39H : Ch.0
				; 3AH : Ch.1
				; 3BH : Ch.2
				; 3CH : Ch.3

;----------------------------------------------------------------------------
; Define I2C Slave Address
;----------------------------------------------------------------------------
PCF8591_ID	EQU	10010000B ; PCF8591:000 Slave Address

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		MOV	P0,#00000000B	; Clear Databus
		SETB	SDA		; CLear I2C bus
		SETB	SCL		;

MAIN:		ACALL	INIT_LCD	; Call LCD Initial subroutine

		MOV	DA_DATA,#0	; Clear D/A Data

MAIN_LOOP:
		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-0FH(Increase automatic)

		MOV	LCD_ADDR,#040H	; Set Address 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_DA	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-0FH(Increase automatic)

		MOV	CHANNEL,#0	; Initial Channel 0
		MOV	R1,#BUFFER	; Set Start Buffer Pointer
		MOV	R4,#4		; Set loop 4 times

CONVERSION_LOOP:MOV	A,CHANNEL	; Get Channel
		ADD	A,#01000000B	; Add Channel to Control Byte
		MOV	CONTROL,A	; Set Control 4 input , Analog Out En.
		ACALL	PCF8591_WR	; Write PCF8591

		ACALL	PCF8591_RD	; Read PCF8591 Repeat Read to
		ACALL	PCF8591_RD	; Read PCF8591 Get Previos Data
		MOV	@R1,AD_DATA	; Read A/D Data to @R1
		INC	R1		; Increase Buffer Pointer
		INC	CHANNEL		; Increase Channel
		DJNZ	R4,CONVERSION_LOOP ; Do until 4 times

		MOV	LCD_ADDR,#04DH	; Set Address 4DH
		ACALL	SET_ADDR_LCD	;
		MOV	LCD_DATA,DA_DATA ; Write D/A Data Out
		ACALL	HEX2LCD		; Show Data as HEX Code

		INC	DA_DATA		; Increase D/A Value

		ACALL	DELAY_1s	; Delay

		MOV	R1,#BUFFER	; Set Start Buffer Pointer

		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_AD01	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-07H,40H-47H(Increase automatic)

		MOV	LCD_ADDR,#040H	; Set Address 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_AD23	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-07H,40H-47H(Increase automatic)

		MOV	LCD_ADDR,#004H	; Set Address 04H
		ACALL	SET_ADDR_LCD	;
		MOV	LCD_DATA,@R1	; Move Data Buffer with Pointer to LCD
		ACALL	HEX2LCD		; Show Data as HEX Code

		INC	R1		; Increase Buffer Pointer

		MOV	LCD_ADDR,#00CH	; Set Address 0CH
		ACALL	SET_ADDR_LCD	;
		MOV	LCD_DATA,@R1	; Move Data Buffer with Pointer to LCD
		ACALL	HEX2LCD		; Show Data as HEX Code

		INC	R1		; Increase Buffer Pointer

		MOV	LCD_ADDR,#044H	; Set Address 44H
		ACALL	SET_ADDR_LCD	;
		MOV	LCD_DATA,@R1	; Move Data Buffer with Pointer to LCD
		ACALL	HEX2LCD		; Show Data as HEX Code

		INC	R1		; Increase Buffer Pointer

		MOV	LCD_ADDR,#04CH	; Set Address 4CH
		ACALL	SET_ADDR_LCD	;
		MOV	LCD_DATA,@R1	; Move Data Buffer with Pointer to LCD
		ACALL	HEX2LCD		; Show Data as HEX Code

		ACALL	DELAY_1s	; Delay

		AJMP	MAIN_LOOP	; Jump to loop

;----------------------------------------------------------------------------
; HEX Code to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
HEX2LCD:	PUSH	ACC		; Push ACC.
		MOV	A,LCD_DATA	; Get Data
		MOV	B,#16		;
		DIV	AB		; Divide by 16
		ADD	A,#030H		; Convert to ASCII
		MOV	LCD_DATA,A	;
		ACALL	HEX_CHK		; Check A-F
		ACALL	WRCHAR_LCD	; Write Higher HEX Code
		MOV	A,B		; Get Remainder
		ADD	A,#030H		; Convert to ASCII
		MOV	LCD_DATA,A	;
		ACALL	HEX_CHK		; Check A-F
		ACALL	WRCHAR_LCD	; Write Lower HEX Code
		POP	ACC		; Pop ACC.
		RET			; Return

HEX_CHK:	MOV	A,LCD_DATA	; Get Data
		CJNE	A,#03AH,CHK_OTHER ; Check Data Equal or Greater than A
CHK_OTHER:	JNC	CONV_2_ALPHA	; If Equal or Greater than => A-F
		RET			; Return

CONV_2_ALPHA:	ADD	A,#7		; Add 7 to Alphabel
		MOV	LCD_DATA,A	; Save Data
		RET			; Return

;----------------------------------------------------------------------------
; Define LCD's command constant
;----------------------------------------------------------------------------
LCD_CMD_CLR	EQU	00000001B	; Clear
LCD_CMD_HOME	EQU	00000010B	; Cursor home
LCD_CMD_AUTOINC	EQU	00000110B	; Auto increment
LCD_CMD_OFF	EQU	00001000B	; Off
LCD_CMD_ON	EQU	00001100B	; On
LCD_CMD_BLINK	EQU	00001111B	; Cursor on & blink
LCD_CMD_LSHF	EQU	00011000B	; Shift display to left
LCD_CMD_RSHF	EQU	00011100B	; Shift display to right


;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:	ACALL   DELAY_100ms	; Delay

		CLR	LCD_RS		; Clear LCD_RS pin
		MOV	P0,#00111000B	; Force 8bit mode on first times
		ACALL	LCD_CLK		; Pulse LCD clock
		ACALL	DELAY_10ms	; Delay

	; force previous possible mode (4 or 8 bit) to be 8 bit
		MOV	P0,#00111000B	; Force 8bit mode again
		ACALL	LCD_CLK		; Pulse LCD clock

		ACALL	LCD_OFF		; Display off
		ACALL	LCD_AUTOINC	; Set auto increment cursor after write
		ACALL	LCD_HOME	; Return cursor to home
		ACALL	LCD_CLR		; Display cleared
		ACALL	LCD_ON		; Display on
		RET

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:	MOV	LCD_DATA,#LCD_CMD_CLR	; Display cleared
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:	MOV	LCD_DATA,#LCD_CMD_HOME	; Return cursor to uppper left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_AUTOINC:	MOV	LCD_DATA,#LCD_CMD_AUTOINC; Cursor auto increment after write
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:	MOV	LCD_DATA,#LCD_CMD_OFF	; Display off
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:		MOV	LCD_DATA,#LCD_CMD_ON	; Display on
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:	MOV	LCD_DATA,#LCD_CMD_BLINK	; Display blink
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Left Shift Display
;----------------------------------------------------------------------------
LCD_LSHF:	MOV	LCD_DATA,#LCD_CMD_LSHF	; Shift display to left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Right Shift Display
;----------------------------------------------------------------------------
LCD_RSHF:	MOV	LCD_DATA,#LCD_CMD_RSHF	; Shift display to right
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:          LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR     LCD_RS		; Clear LCD_RS pin
		MOV     P0,LCD_ADDR	; Move LCD_ADDR to P0
		ORL	P0,#10000000B	; Set MSB of P0
		ACALL   LCD_CLK		; Pulse LCD clock
		RET			; Return

;----------------------------------------------------------------------------
; LCD Clear Display
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCMD_LCD:	CLR	LCD_RS		; Clear LCD_RS Pin
		MOV	P0,LCD_DATA		; Write command to LCD
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET			; Return

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:	SETB	LCD_RS		; Set LCD_RS Pin
		MOV	P0,LCD_DATA	; Move LCD_DATA to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET						; Return

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:	MOV	R0,#0		; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		INC	R0		; Increase loop counter
		CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
		RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:	SETB	LCD_EN		; Pulse Clock to LCD_EN
		ACALL	LCD_DELAY
		CLR	LCD_EN
		ACALL	LCD_DELAY
		RET

;----------------------------------------------------------------------------
; I2C PCF8591 Read
;----------------------------------------------------------------------------
PCF8591_RD:	MOV	I2C_ADDR,#PCF8591_ID+1 ; Set PCF8591 as I2C Read Slave
		ACALL	I2C_SLAVE	; Connect Slave

		ACALL	I2C_DATA_RD	; Read Data from Slave
		MOV	AD_DATA,I2C_DATA ; Read Data to A/D Data
		ACALL	I2C_NACK_BIT	; Send Not Acknowledge

		ACALL	I2C_STOP	; Send Stop Condition
		RET			; Return

;----------------------------------------------------------------------------
; I2C PCF8591 Write
;----------------------------------------------------------------------------
PCF8591_WR:	MOV	I2C_ADDR,#PCF8591_ID ; Set PCF8591 as I2C Write Slave
		ACALL	I2C_SLAVE	; Connect Slave

		MOV	I2C_DATA,CONTROL ; Write CONTROL to Slave
		ACALL	I2C_DATA_WR	; Write Data to Slave

		MOV	I2C_DATA,DA_DATA ; Write D/A Data to Slave
		ACALL	I2C_DATA_WR	; Write Data to Slave

		ACALL	I2C_STOP	; Send Stop Condition
		RET			; Return

;----------------------------------------------------------------------------
; I2C Data Write
; I/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_WR:	PUSH	ACC		; Push ACC.
		SETB	I2C_ACK		; Set ACK. bit
		MOV	A,I2C_DATA	; Get Data
		MOV	R5,#008		; Set loop 8 times
I2C_DATA_WR_1:	RLC	A		; Rotate ACC. to Left with Carry
		MOV	SDA,C		; Move Carry Flag to SDA
		ACALL	I2C_CLK		; Pulse I2C Clock
		DJNZ	R5,I2C_DATA_WR_1; Do until 8 times
		SETB	SDA		; Set SDA
		ACALL	I2C_DELAY	; Delay
		SETB	SCL		; Set SCL
		ACALL	I2C_DELAY	; Delay
		JB	SDA,I2C_DATA_WR_2; Check Acknowledge from Slave
		CLR	I2C_ACK		; Clear ACK. bit
I2C_DATA_WR_2:	CLR	SCL		; Clear SCL
		POP	ACC		; Pop ACC.
		RET			; Return

;----------------------------------------------------------------------------
; I2C Data Read
; O/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_RD:	PUSH	ACC		; Push ACC.
		CLR	A		; Clear ACC.
		MOV	R5,#008		; Set loop 8 times
I2C_DATA_RD_1:	ACALL	I2C_DELAY	; Delay
		SETB	SCL		; Set SCL
		ACALL	I2C_DELAY	; Delay
		MOV	C,SDA		; Get SDA to Carry Flag
		RLC	A		; Rotate ACC. to Left with Carry
		CLR	SCL		; Clear SCL
		DJNZ	R5,I2C_DATA_RD_1; Do until 8 times
		MOV	I2C_DATA,A	; Move Data to I2C_DATA
		POP	ACC		; Pop ACC.
		RET			; Return

;----------------------------------------------------------------------------
; I2C Slave Connect
; I/P:		I2C_ADDR
; O/P Flag:	I2C_ACK
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_SLAVE:	PUSH	ACC		; Push ACC.
		SETB	I2C_ACK		; Set ACK. bit
		MOV	A,I2C_ADDR	; Get Slave Address
		ACALL	I2C_START	; Send Start Condition
		MOV	R5,#008		; Set loop 8 times
I2C_SLAVE_1:	RLC	A		; Rotate ACC. to Left with Carry
		MOV	SDA,C		; Move Carry Flag to SDA
		ACALL	I2C_CLK		; Pulse I2C Clock
		DJNZ	R5,I2C_SLAVE_1	; Do until 8 times

		SETB	SDA		; Set SDA
		ACALL	I2C_DELAY	; Delay
		SETB	SCL		; Set SCL
		ACALL	I2C_DELAY	; Delay
		JB	SDA,I2C_SLAVE_2 ; Check Acknowledge from Slave
		CLR	I2C_ACK		; Clear ACK.
I2C_SLAVE_2:	CLR	SCL		; Clear SCL
		POP	ACC		; Pop ACC.
		RET			; Return

;----------------------------------------------------------------------------
; I2C Start Condition
;----------------------------------------------------------------------------
I2C_START:	JNB	SCL,I2C_START_1	; Check current SCL set?
		CLR	SCL		; Clear SCL

I2C_START_1:	SETB	SDA		; Set SDA
		SETB	SCL		; Set SCL

		ACALL	I2C_DELAY	; Delay
		CLR	SDA		; Clear SDA during SCL set
		ACALL	I2C_DELAY	; Delay
		CLR	SCL		; Clear SCL
		RET			; Return

;----------------------------------------------------------------------------
; I2C Stop Condition
;----------------------------------------------------------------------------
I2C_STOP:	JNB	SCL,I2C_STOP_1	; Check current SCL set?
		CLR	SCL		; Clear SCL

I2C_STOP_1:	CLR	SDA		; Clear SDA
		ACALL	I2C_DELAY	; Delay
		SETB	SCL		; Set SCL
		ACALL	I2C_DELAY	; Delay
		SETB	SDA		; Set SDA during SCL set
		RET			; Return

;----------------------------------------------------------------------------
; I2C Clock
;----------------------------------------------------------------------------
I2C_CLK:	ACALL	I2C_DELAY	; Pulse SCL
		SETB	SCL		;
		ACALL	I2C_DELAY	;
		CLR	SCL		;
		RET			; Return

;----------------------------------------------------------------------------
; I2C Acknowledge
;----------------------------------------------------------------------------
I2C_ACK_BIT:	CLR	SDA		; Clear SDA
		ACALL	I2C_DELAY	; Delay
		ACALL	I2C_CLK		; Pulse I2C Clock
		SETB	SDA
		RET			; Return

;----------------------------------------------------------------------------
; I2C Not Acknowledge
;----------------------------------------------------------------------------
I2C_NACK_BIT:	SETB	SDA		; Set SDA
		ACALL	I2C_DELAY	; Delay
		ACALL	I2C_CLK		; Pulse I2C Clock
		SETB	SCL
		RET			; Return

;----------------------------------------------------------------------------
; Dummy Delay time I2C_DELAY, LCD_DELAY, 10ms, 100m, 1s
;----------------------------------------------------------------------------
I2C_DELAY:	MOV	R6,#00CH	; Each loop = 50 us
I2C_DELAY_1:	NOP
		NOP
		DJNZ	R6,I2C_DELAY_1
		RET

LCD_DELAY:	MOV	R7,#002		; Do 2 times
LCD_DELAY_1:	MOV	R6,#0E6H	; Each loop = 1 ms
LCD_DELAY_2:	NOP
		NOP
		DJNZ	R6,LCD_DELAY_2
		DJNZ	R7,LCD_DELAY_1
		RET

DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_100ms:	MOV	R7,#100		; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

DELAY_1s:	MOV	R5,#100		; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;			 0123456789ABCDEF
TITLE_1:	DB	'I2C 8591 A/D D/A'
SCR_AD01:	DB	'Ch0:  H Ch1:  H '
SCR_AD23:	DB	'Ch2:  H Ch3:  H '
SCR_DA:		DB	'Analog Out :   H'

		END