;****************************************************************************
; Program	: Temperature DS18S20 01
; Description	: Temperature DS18S20 1-Wire Reading (0-127.5C)
;		: work both External power and Parasite power
; For		: MCS-51 Microcontroller Training System
; Filename	: lab2101.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
ONEWIRE		BIT	P2.7		; 1-Wire Interface (DS1820 Temp. Sensor)

LCD_EN		BIT	P3.6		; LCD Module Enable (Active High : Level)
LCD_RS		BIT	P3.7		; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG		EQU	02FH		; User FLAG
BUSY		BIT	FLAG.0		; Define BUSY as bit
DS1820_ERROR	BIT	FLAG.1		; DS1820 error flag

LCD_ADDR	EQU	030H		; For keep LCD Address
LCD_DATA	EQU	031H		; For keep LCD Data
ONEWIRE_DATA	EQU	032H		; For keep ONEWIRE Data
TEMP		EQU	033H		; For keep Temp. Data (Temp L only)
TEMPH		EQU	034H		; For keep Temp. Data (Temp H only)
ONEWIRE_CNT	EQU	035H		; For 1-Wire subroutine counter

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
INITIAL:	ACALL	INIT_LCD	; Call LCD Initial subroutine

		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-0FH(Increase automatic)

		MOV	LCD_ADDR,#040H	; Set Address 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_TEMP	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 40H-4FH(Increase automatic)

MAIN:		ACALL	DS1820_TEMP_RD	; Read DS1820

		MOV	LCD_ADDR,#048H	; Set Address 48H
		ACALL	SET_ADDR_LCD	;

		JB	DS1820_ERROR,DATA_INVALID	; Check device is present?

		MOV	A,#0AAH		; Check data not ready (AAH)
		CJNE	A,TEMP,DATA_VALID
DATA_INVALID:	MOV	LCD_DATA,#'-'	; Not set => Load '-'
		ACALL	WRCHAR_LCD	; Write to LCD
		MOV	LCD_DATA,#'-'	; Not set => Load '-'
		ACALL	WRCHAR_LCD	; Write to LCD
		MOV	LCD_DATA,#'-'	; Not set => Load '-'
		ACALL	WRCHAR_LCD	; Write to LCD
		MOV	LCD_ADDR,#04CH	; Set Address 4CH
		ACALL	SET_ADDR_LCD	;
		MOV	LCD_DATA,#'-'	; Not set => Load '-'
		ACALL	WRCHAR_LCD	; Write to LCD

		AJMP	MAIN		; Back to main again

DATA_VALID:	MOV	A,TEMP		; Get Temp. Data
		CLR	C		; Clear Carry Flag
		RRC	A		; Rotate ACC. to Right with Carry

		MOV	LCD_DATA,A	; Write Temp. in 3 Decimal Digits
		ACALL	BYTE2LCD	; Show byte value as 3 digit decimal

		MOV	LCD_ADDR,#04CH	; Set Address 4CH
		ACALL	SET_ADDR_LCD	;

		MOV	A,TEMP		; Get Temp. Data
		JNB	ACC.0,WRITE_0C	; Check LSB was set?
		MOV	LCD_DATA,#'5'	; Set => Load '5'
		AJMP	WRITE_NEXT	; Jump to write LCD
WRITE_0C:	MOV	LCD_DATA,#'0'	; Not set => Load '0'
WRITE_NEXT:	ACALL	WRCHAR_LCD	; Write to LCD
		ACALL	DELAY_1s	; Delay
		AJMP	MAIN		; Jump to loop

;----------------------------------------------------------------------------
; Show byte value as 3 digit decimal
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
BYTE2LCD:	PUSH	ACC		; Push ACC.
		MOV	A,LCD_DATA	; Get Data
		MOV	B,#100		;
		DIV	AB		; Divide by 100
		ADD	A,#030H		; Convert to ASCII
		CJNE	A,#030H,BYTE2LCD_1 ; Check x100 = 0 ?
		MOV	A,#' '		; 0 => Write Space
BYTE2LCD_1:	MOV	LCD_DATA,A	;
		ACALL	WRCHAR_LCD	; Write x100
		MOV	A,B		;
		MOV	B,#10		;
		DIV	AB		; Divide by 10
		ADD	A,#030H		; Convert to ASCII
		MOV	LCD_DATA,A	;
		ACALL	WRCHAR_LCD	; Write Lower HEX Code
		MOV	A,B		; Get Remainder x1
		ADD	A,#030H		; Convert to ASCII
		MOV	LCD_DATA,A	;
		ACALL	WRCHAR_LCD	; Write Lower HEX Code
		POP	ACC		; Pop ACC.
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Temperature Read
;----------------------------------------------------------------------------
DS1820_TEMP_RD: 
                CLR	DS1820_ERROR	; Clear DS1820 error flag

                ACALL   DS1820_RST      ; DS1820 Reset
                ACALL   DS1820_PRES     ; DS1820 Presence
                MOV     ONEWIRE_DATA,#0CCH ; Write Skip ROM
                ACALL   DS1820_WR       ;
                MOV     ONEWIRE_DATA,#044H ; Write Convert Command
                ACALL   DS1820_WR       ;

		SETB	ONEWIRE		; Hold logic 1 (conversion)
		ACALL	DELAY_1s	; for 1s.

		ACALL   DS1820_RST      ; DS1820 Reset
                ACALL   DS1820_PRES     ; DS1820 Presence
                MOV     ONEWIRE_DATA,#0CCH ; Write Skip ROM
                ACALL   DS1820_WR
                MOV     ONEWIRE_DATA,#0BEH ; Write Read Scratchpad Command
                ACALL   DS1820_WR

		ACALL   DS1820_RD      	; Read DS1820
                MOV     TEMP,ONEWIRE_DATA; Get byte TEMPL

		ACALL   DS1820_RD      	; Read DS1820
                MOV     TEMPH,ONEWIRE_DATA; Get byte TEMPH

DS1820_TEMP_CLR:
                ACALL   DS1820_RST      ; DS1820 Reset
                ACALL   DS1820_PRES     ; DS1820 Presence

DS1820_TEMP_EXIT:
                RET

;----------------------------------------------------------------------------
; DS1820 Data Read
;----------------------------------------------------------------------------
DS1820_RD:	MOV	ONEWIRE_CNT,#8		; Set loop 8 times
		CLR	A		; Clear ACC.
DS1820_RD_LOOP: CLR	ONEWIRE		; Clear ONEWIRE
		NOP			; Delay
		SETB	ONEWIRE		; Set ONEWIRE
		NOP			;
		NOP			;
		MOV	C,ONEWIRE	; Get ONEWIRE to Carry Flag
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		RRC	A		;
		DJNZ	ONEWIRE_CNT,DS1820_RD_LOOP ; Do until 8 times
		MOV	ONEWIRE_DATA,A	; Move ACC. to ONEWIRE_DATA
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Data Write
;----------------------------------------------------------------------------
DS1820_WR:	MOV	ONEWIRE_CNT,#8		; Set loop 8 times
		MOV	A,ONEWIRE_DATA	; Get ONEWIRE_DATA
DS1820_WR_LOOP: RRC	A		; Rotate ACC. with Carry Flag
		JNC	DS1820_WR_L	; Carry Flag was set ?
		CLR	ONEWIRE		; Set => TX high
		NOP			; Delay
		NOP			;
		SETB	ONEWIRE		; Set ONEWIRE
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		AJMP	DS1820_WR_NX	; Jump to next write
DS1820_WR_L:	CLR	ONEWIRE		; Clear => TX Low
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		SETB	ONEWIRE		; Set ONEWIRE
		NOP			; Delay
		NOP			;
DS1820_WR_NX:	DJNZ	ONEWIRE_CNT,DS1820_WR_LOOP ; Do until 8 times
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Reset
;----------------------------------------------------------------------------
DS1820_RST:	CLR	ONEWIRE		; Clear ONEWIRE
		ACALL	DELAY_750us	; Delay
		SETB	ONEWIRE		; Set ONEWIRE
		MOV	ONEWIRE_CNT,#11	; Delay
		DJNZ	ONEWIRE_CNT,$	;
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Receive Presence Pulse
;----------------------------------------------------------------------------
DS1820_PRES:	MOV	ONEWIRE_CNT,#55	; Set Loop wait 1
DS1820_PRES_1:	JNB	ONEWIRE,DS1820_PRES_2 ; Check ONEWIRE was clear?
		DJNZ	ONEWIRE_CNT,DS1820_PRES_1 ; Wait loop check 1
                SETB    DS1820_ERROR		; Error if no response
		RET			; Return

DS1820_PRES_2:	MOV     ONEWIRE_CNT,#55	; Set Loop wait 2
DS1820_PRES_3:	JB	ONEWIRE,DS1820_PRES_4	; Check ONEWIRE was clear?
                DJNZ    ONEWIRE_CNT,DS1820_PRES_3	; Wait loop check 1
                SETB    DS1820_ERROR		; Error if no response
DS1820_PRES_4:	SETB	ONEWIRE		; Set ONEWIRE
                ACALL   ONEWIRE_DELAY	; Delay 75 us
		CLR	BUSY		; Clear BUSY Flag
		RET			; Return

;----------------------------------------------------------------------------
; Define LCD's command constant
;----------------------------------------------------------------------------
LCD_CMD_CLR	EQU	00000001B	; Clear
LCD_CMD_HOME	EQU	00000010B	; Cursor home
LCD_CMD_AUTOINC	EQU	00000110B	; Auto increment
LCD_CMD_OFF	EQU	00001000B	; Off
LCD_CMD_ON	EQU	00001100B	; On
LCD_CMD_BLINK	EQU	00001111B	; Cursor on & blink
LCD_CMD_LSHF	EQU	00011000B	; Shift display to left
LCD_CMD_RSHF	EQU	00011100B	; Shift display to right


;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:	ACALL   DELAY_100ms	; Delay

		CLR	LCD_RS		; Clear LCD_RS pin
		MOV	P0,#00111000B	; Force 8bit mode on first times
		ACALL	LCD_CLK		; Pulse LCD clock
		ACALL	DELAY_10ms	; Delay

	; force previous possible mode (4 or 8 bit) to be 8 bit
		MOV	P0,#00111000B	; Force 8bit mode again
		ACALL	LCD_CLK		; Pulse LCD clock

		ACALL	LCD_OFF		; Display off
		ACALL	LCD_AUTOINC	; Set auto increment cursor after write
		ACALL	LCD_HOME	; Return cursor to home
		ACALL	LCD_CLR		; Display cleared
		ACALL	LCD_ON		; Display on
		RET

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:	MOV	LCD_DATA,#LCD_CMD_CLR	; Display cleared
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:	MOV	LCD_DATA,#LCD_CMD_HOME	; Return cursor to uppper left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_AUTOINC:	MOV	LCD_DATA,#LCD_CMD_AUTOINC; Cursor auto increment after write
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:	MOV	LCD_DATA,#LCD_CMD_OFF	; Display off
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:		MOV	LCD_DATA,#LCD_CMD_ON	; Display on
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:	MOV	LCD_DATA,#LCD_CMD_BLINK	; Display blink
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Left Shift Display
;----------------------------------------------------------------------------
LCD_LSHF:	MOV	LCD_DATA,#LCD_CMD_LSHF	; Shift display to left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Right Shift Display
;----------------------------------------------------------------------------
LCD_RSHF:	MOV	LCD_DATA,#LCD_CMD_RSHF	; Shift display to right
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:          LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR     LCD_RS		; Clear LCD_RS pin
		MOV     P0,LCD_ADDR	; Move LCD_ADDR to P0
		ORL	P0,#10000000B	; Set MSB of P0
		ACALL   LCD_CLK		; Pulse LCD clock
		RET			; Return

;----------------------------------------------------------------------------
; LCD Clear Display
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCMD_LCD:	CLR	LCD_RS		; Clear LCD_RS Pin
		MOV	P0,LCD_DATA		; Write command to LCD
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET			; Return

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:	SETB	LCD_RS		; Set LCD_RS Pin
		MOV	P0,LCD_DATA	; Move LCD_DATA to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET						; Return

;----------------------------------------------------------------------------
; Put string from ROM (string must ended with 000H)
; I/P:          DPTR : Locate ROM Address
;----------------------------------------------------------------------------
STRING_LCD:	PUSH	ACC		; Put ACC. into stack
STRING_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		CJNE	A,#0,STRING_LCD_2	; Do until null terminate detect
		POP	ACC		; Pop ACC from stack
		RET			; Return
				
STRING_LCD_2:	MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		AJMP	STRING_LCD_1

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:	MOV	R0,#0		; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		INC	R0		; Increase loop counter
		CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
		RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:	SETB	LCD_EN		; Pulse Clock to LCD_EN
		ACALL	LCD_DELAY
		CLR	LCD_EN
		ACALL	LCD_DELAY
		RET

;----------------------------------------------------------------------------
; Dummy Delay time ONEWIRE_DELAY, LCD_DELAY, 50u, 100u, 1m, 10m, 100m, 1s
;----------------------------------------------------------------------------
ONEWIRE_DELAY:	MOV	R6,#012H		; Each loop = 75 us
ONEWIRE_DELAY_1: NOP
		NOP
		DJNZ	R6,ONEWIRE_DELAY_1
		RET

LCD_DELAY:	MOV	R7,#002			; Do 2 times
LCD_DELAY_1:	MOV	R6,#0E6H		; Each loop = 1 ms
LCD_DELAY_2:	NOP
		NOP
		DJNZ	R6,LCD_DELAY_2
		DJNZ	R7,LCD_DELAY_1
		RET

DELAY_50us:	MOV	R6,#00CH		; Each loop = 50 us
DELAY_50us_1:	NOP
		NOP
		DJNZ	R6,DELAY_50us_1
		RET

DELAY_100us:	MOV	R6,#017H		; Each loop = 100 us
DELAY_100us_1:	NOP
		NOP
		DJNZ	R6,DELAY_100us_1
		RET

DELAY_1ms:	MOV	R6,#0E6H		; Each loop = 1 ms
DELAY_1ms_1:	NOP
		NOP
		DJNZ	R6,DELAY_1ms_1
		RET
				
DELAY_10ms:	MOV	R7,#010			; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_100ms:	MOV	R7,#100			; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

DELAY_1s:	MOV	R5,#100			; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

DELAY_750us:	MOV     R6,#229		; Each loop = 750us
DELAY_750us_2:	NOP
		DJNZ    R6,DELAY_750us_2
                RET				; Return

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;			 0123456789ABCDEF
TITLE_1:	DB	' 1-Wire DS18S20 '
SCR_TEMP:	DB	' Temp :    . ',0DFH,'C '	; 0DFH = degree symbol

		END