;****************************************************************************
; Program       : Temperature DS18S20 02
; Description	: Temperature DS18S20 1-Wire high resolution reading (2 decimal points)
;		: work both External power and Parasite power
; For		: MCS-51 Microcontroller Training System
; Filename	: lab2102.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************
;
;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
ONEWIRE		BIT	P2.7		; 1-Wire port

LCD_EN		BIT	P3.6		; LCD module enable (active high : level)
LCD_RS		BIT	P3.7		; LCD module register select

;----------------------------------------------------------------------------
; Define flag
;----------------------------------------------------------------------------
FLAG		EQU	02FH		; Alias FLAG @02FH
BUSY		BIT	FLAG.0		; Busy flag
DS1820_ERROR	BIT	FLAG.1		; DS1820 error flag

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
ONEWIRE_DATA	EQU	030H		; 1-Wire data
TEMP		EQU	031H		; Temperature low [Start of array]
TEMPH		EQU	032H		; Temperature high
USER1		EQU	033H		; TH/USER 1
USER2		EQU	034H		; TL/USER 2
RSV1		EQU	035H		;
RSV2		EQU	036H		;
CNT_REMAIN	EQU	037H		; Count remain
CNT_PER_C	EQU	038H		; Count per C
CRC_RECV	EQU	039H		; CRC received from 1-Wire
CRC		EQU	03AH		; CRC calculate from data
TEMP_CAL	EQU	03BH		; Temperature calculate
TEMP_DOT	EQU	03CH		; Temperature calculate 2 decimal
LCD_ADDR	EQU	03DH		; For keep LCD address
LCD_DATA	EQU	03EH		; For keep LCD data

ONEWIRE_LOOP	EQU	03FH		; Alias 1-Wire loop counter @03FH
ONEWIRE_CNT	EQU	040H		; Alias 1-Wire counter @040H

; Local variable for unsigned 16 bit multiply subroutine
O4		EQU	041H
O5		EQU	042H
O6		EQU	043H
O7		EQU	044H
OL		EQU	045H
OH		EQU	046H
; Input/Output variable
ACCL		EQU	047H
ACCH		EQU	048H
OPRL		EQU	049H
OPRH		EQU	04AH

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
INITIAL:	ACALL	INIT_LCD	; Call initial LCD subroutine

		MOV	LCD_ADDR,#0	; Set LCD at 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE	; Write title
		ACALL	STRING_LCD	;

		MOV	LCD_ADDR,#040H	; Set LCD at 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#STRING	; Put string
		ACALL	STRING_LCD	;


MAIN:		ACALL	DS1820_TEMP_RD	; Read for DS1820

 		MOV	R0,#TEMP	; Calculate CRC
		MOV	R2,#8		;
		MOV	CRC,#0		; Initial CRC=0
BUFFER_READ:	MOV     A,@R0		; Calculate each byte
		ACALL	CRC8		; Into CRC
		INC	R0		;
		DJNZ	R2,BUFFER_READ	; Do until 8 times

		MOV	A,CRC		; Compare with receive CRC
		CJNE	A,CRC_RECV,ERROR_CRC ; Error if not equal

		MOV	A,#0AAH		; Check data not ready (AAH)
		CJNE	A,TEMP,DATA_VALID
		MOV	LCD_ADDR,#048H	; Show initial message start @048H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#INIT_MSG	;
		ACALL	STRING_LCD	;
		AJMP	MAIN		; Back to main again

DATA_VALID:	MOV	A,CNT_REMAIN	; Calculate addition 2 digits decimal point
		MOV	B,#100		; 2 digits addition = 75-(100(CNT_REMAIN/CNT_PER_C))
		MUL	AB		;

		MOV	ACCL,A		; Load lower byte of dividend
		MOV	ACCH,B		; Load higher byte of dividend
		MOV	OPRL,CNT_PER_C	; Load lower byte of divisor
		MOV	OPRH,#0		; Load higher byte of divisos = 0

		ACALL	UDIV16		; Call unsigned division 16bit

		MOV	TEMP_CAL,ACCL	; Get result (100(CNT_REMAIN/CNT_PER_C))

		MOV	A,#75		; Subtract 75 with last result
		CLR	C		;
		SUBB	A,TEMP_CAL	;
		JNC	SHOW_TEMP	; Negative result?

MINUS:		CPL	A		; Complement result
		INC	A		; Get number
		MOV	B,A		;
		MOV	A,#100		; 100-(complemented result)
		CLR	C		;
		SUBB	A,B		;
		DEC	TEMP		; Decrease TEMP

SHOW_TEMP:	MOV	TEMP_DOT,A	; Store 2 digits answer

		MOV	LCD_ADDR,#048H	; Show temp. start @048H
		ACALL	SET_ADDR_LCD	;
		MOV	A,TEMP		;
		CLR	C		;
		RRC	A		; Reject 0.5 C bit
		MOV	LCD_DATA,A	; Show TEMP on LCD as 3 digits decimal number
		ACALL	DATA2LCD		;

		MOV	LCD_DATA,#"."	; Show "."
		ACALL	WRCHAR_LCD	;

		MOV	LCD_DATA,TEMP_DOT; Show addition 2 digit decimal point
		ACALL	DATA2LCD		;

		MOV	DPTR,#UNIT_MSG	; Suffix unit addition
		ACALL	STRING_LCD	;

		ACALL	DELAY_1s	; Delay 1s

		AJMP	MAIN		; Jump to main

ERROR_CRC:	MOV	LCD_ADDR,#048H	; Show error message start @048H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#CRC_ERR_MSG
		ACALL	STRING_LCD	;
		AJMP	MAIN		; Jump to main

;----------------------------------------------------------------------------
; Show byte value as 2 digit decimal
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
DATA2LCD:	PUSH	ACC		; Push ACC. To Stack
		PUSH	B		; Push B to Stack
		MOV	A,LCD_DATA	; Get input data value
		MOV	B,#100		; Copy to B
		DIV	AB
		MOV	A,B
		MOV	B,#10
		DIV	AB
		ADD	A,#030H		; Convert to ASCII
		MOV	LCD_DATA,A	; Write LCD
		ACALL	WRCHAR_LCD	;
		MOV	A,B
		ADD	A,#030H		; Convert to ASCII
		MOV	LCD_DATA,A	; Write LCD
		ACALL	WRCHAR_LCD	;
		POP	B		; Pop B from Stack
		POP	ACC		; Pop ACC. from Stack
		RET			; Return

;----------------------------------------------------------------------------
; Divide 16 bit by 16 bit
; IN
;	ACCL: Dividen low byte
;	ACCH: Dividen high byte
;	OPRL: Divisor low byte
;	OPRH: Divisor high byte
; OUT
;	ACCL: Quotient low byte
;	ACCH: Quotient high bye
;	OPRL: Remainder low byte
;	OPRH: Remainder high byte
;----------------------------------------------------------------------------
UDIV16:		PUSH	ACC		; Push ACC. into stack
		PUSH	B		; Push B into stack

		MOV	O7,#0		; Clear remainder
		MOV	O6,#0
		MOV	B,#16		; Set 16 bit loop counter
UDIV16_LOOP:	CLR	C		; Clear carry flag
		MOV	A,ACCL		; Shift low byte dividend
		RLC	A
		MOV	ACCL,A
		MOV	A,ACCH		; Shift high byte dividend
		RLC	A
		MOV	ACCH,A
		MOV	A,O6		; Shift low byte remainder
		RLC	A
		MOV	O6,A
		MOV	A,O7		; Shift high byte remainder
		RLC	A
		MOV	O7,A
		MOV	A,O6		; Trial subtract divisor from remainder
		CLR	C
		SUBB	A,OPRL
		MOV	OL,A
		MOV	A,O7
		SUBB	A,OPRH
		MOV	OH,A
		CPL	C		; Complement borrow
		JNC	UDIV16_1	; Check borrow
		MOV	O7,OH
		MOV	O6,OL
UDIV16_1:	MOV	A,O4		; Shift result to low byte quotient
		RLC	A
		MOV	O4,A
		MOV	A,O5		; Shift result to high byte quotient
		RLC	A
		MOV	O5,A
		DJNZ	B, UDIV16_LOOP
		MOV	A,O5		; Replace ACCL ACCH with quotient
		MOV	ACCH,A
		MOV	A,O4
		MOV	ACCL,A
		MOV	A,O7		; Replace OPRL OPRH with remainder
		MOV	OPRH,A
		MOV	A,O6
		MOV	OPRL,A

		POP	B		; Pop B from stack
		POP	ACC		; Pop ACC. from stack
		RET			; Return to main

;----------------------------------------------------------------------------
; CRC-8 CCITT CRC 8 calculate
;----------------------------------------------------------------------------
CRC8:		PUSH	ACC		; Push ACC. into stack
		PUSH	B		; Push B into stack
		PUSH	ACC		; Push ACC. into stack
		MOV	B,#8		; Set loop 8 times
CRC8_LOOP:	XRL	A,CRC		; Exclusive OR CRC data with data
		RRC	A		; Get bit
		MOV	A,CRC		;
		JNC	CRC_BIT_ZERO	; Check bit
		XRL	A,#18H		; Exclusive OR with CRC-8 expression
CRC_BIT_ZERO:	RRC	A		; Get bit
		MOV	CRC,A		; Replace CRC
		POP	ACC		; Pop ACC. from stach
		RR	A		; Rotate to next bit
		PUSH	ACC		; Push ACC. into stack
		DJNZ	B,CRC8_LOOP	; Do loop
		POP	ACC		; Pop ACC. from stack
		POP	B		; Pop B from stack
		POP	ACC		; Pop ACC. from stack
		RET

;----------------------------------------------------------------------------
; DS1820 Temperature Read
;----------------------------------------------------------------------------
DS1820_TEMP_RD:
		CLR	DS1820_ERROR	; Clear DS1820 error flag

		ACALL	DS1820_RST	; DS1820 Reset
		ACALL	DS1820_PRES	; DS1820 Presence
		MOV	ONEWIRE_DATA,#0CCH ; Write Skip ROM
		ACALL	DS1820_WR	;
		MOV	ONEWIRE_DATA,#044H ; Write Convert Command
		ACALL	DS1820_WR	;

		SETB	ONEWIRE		; Hold logic 1 (conversion)
		ACALL	DELAY_1s	; for 1s.

		ACALL   DS1820_RST	; DS1820 Reset
		ACALL	DS1820_PRES	; DS1820 Presence
		MOV	ONEWIRE_DATA,#0CCH ; Write Skip ROM
		ACALL	DS1820_WR
		MOV	ONEWIRE_DATA,#0BEH ; Write Read Scratchpad Command
		ACALL	DS1820_WR

		MOV	R0,#TEMP	; Point to TEMP
		MOV	ONEWIRE_LOOP,#9	; Read 9 byte
DS1820_TEMP_LOOP:
		ACALL	DS1820_RD	; Read DS1820
		MOV	@R0,ONEWIRE_DATA; Get byte from TEMPL to CRC_RECV
		INC	R0		;
		DJNZ	ONEWIRE_LOOP,DS1820_TEMP_LOOP	;

DS1820_TEMP_CLR:
		ACALL	DS1820_RST	; DS1820 Reset
		ACALL	DS1820_PRES	; DS1820 Presence

DS1820_TEMP_EXIT:
		RET

;----------------------------------------------------------------------------
; DS1820 Data Read
;----------------------------------------------------------------------------
DS1820_RD:	MOV	ONEWIRE_CNT,#8	; Set loop 8 times
		CLR	A		; Clear ACC.
DS1820_RD_LOOP:	CLR	ONEWIRE		; Clear ONEWIRE
		NOP			; Delay
		SETB	ONEWIRE		; Set ONEWIRE
		NOP			; Delay
		NOP			;
		MOV	C,ONEWIRE	; Get ONEWIRE to Carry Flag
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		RRC	A		;
		SETB	ONEWIRE		; Set ONEWIRE
		DJNZ	ONEWIRE_CNT,DS1820_RD_LOOP ; Do until 8 times
		MOV	ONEWIRE_DATA,A	; Move ACC. to ONEWIRE_DATA
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Data Write
;----------------------------------------------------------------------------
DS1820_WR:	MOV	ONEWIRE_CNT,#8	; Set loop 8 times
		MOV	A,ONEWIRE_DATA	; Get ONEWIRE_DATA
DS1820_WR_LOOP:	RRC	A		; Rotate ACC. with Carry Flag
		JNC	DS1820_WR_L	; Carry Flag was set ?
		CLR	ONEWIRE		; Set => TX high
		NOP			; Delay
		NOP			;
		SETB	ONEWIRE		; Set ONEWIRE
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		AJMP	DS1820_WR_NX	; Jump to next write
DS1820_WR_L:	CLR	ONEWIRE		; Clear => TX Low
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		SETB	ONEWIRE		; Set ONEWIRE
		NOP			; Delay
		NOP			;
DS1820_WR_NX:	DJNZ	ONEWIRE_CNT,DS1820_WR_LOOP ; Do until 8 times
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Reset
;----------------------------------------------------------------------------
DS1820_RST:	CLR	ONEWIRE		; Clear ONEWIRE
		ACALL	DELAY_750us	; Delay
		SETB	ONEWIRE		; Set ONEWIRE
		MOV	ONEWIRE_CNT,#11	; Delay
		DJNZ	ONEWIRE_CNT,$	;
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Receive Presence Pulse
;----------------------------------------------------------------------------
DS1820_PRES:	MOV	ONEWIRE_CNT,#110	; Set Loop wait 1
DS1820_PRES_1:	JNB	ONEWIRE,DS1820_PRES_2	; Check ONEWIRE was clear?
		DJNZ	ONEWIRE_CNT,DS1820_PRES_1	; Wait loop check 1
		SETB	DS1820_ERROR	; Error if no response
		RET			; Return

DS1820_PRES_2:	MOV	ONEWIRE_CNT,#110 ; Set Loop wait 2
DS1820_PRES_3:	JB	ONEWIRE,DS1820_PRES_4	; Check ONEWIRE was clear?
		DJNZ	ONEWIRE_CNT,DS1820_PRES_3	; Wait loop check 1
		SETB	DS1820_ERROR	; Error if no response
DS1820_PRES_4:	SETB	ONEWIRE		; Set ONEWIRE
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		RET			; Return

;----------------------------------------------------------------------------
; Define LCD's command constant
;----------------------------------------------------------------------------
LCD_CMD_CLR	EQU	00000001B	; Clear
LCD_CMD_HOME	EQU	00000010B	; Cursor home
LCD_CMD_AUTOINC	EQU	00000110B	; Auto increment
LCD_CMD_OFF	EQU	00001000B	; Off
LCD_CMD_ON	EQU	00001100B	; On
LCD_CMD_BLINK	EQU	00001111B	; Cursor on & blink
LCD_CMD_LSHF	EQU	00011000B	; Shift display to left
LCD_CMD_RSHF	EQU	00011100B	; Shift display to right


;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:	ACALL   DELAY_100ms	; Delay

		CLR	LCD_RS		; Clear LCD_RS pin
		MOV	P0,#00111000B	; Force 8bit mode on first times
		ACALL	LCD_CLK		; Pulse LCD clock
		ACALL	DELAY_10ms	; Delay

	; force previous possible mode (4 or 8 bit) to be 8 bit
		MOV	P0,#00111000B	; Force 8bit mode again
		ACALL	LCD_CLK		; Pulse LCD clock

		ACALL	LCD_OFF		; Display off
		ACALL	LCD_AUTOINC	; Set auto increment cursor after write
		ACALL	LCD_HOME	; Return cursor to home
		ACALL	LCD_CLR		; Display cleared
		ACALL	LCD_ON		; Display on
		RET

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:	MOV	LCD_DATA,#LCD_CMD_CLR	; Display cleared
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:	MOV	LCD_DATA,#LCD_CMD_HOME	; Return cursor to uppper left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_AUTOINC:	MOV	LCD_DATA,#LCD_CMD_AUTOINC; Cursor auto increment after write
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:	MOV	LCD_DATA,#LCD_CMD_OFF	; Display off
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:		MOV	LCD_DATA,#LCD_CMD_ON	; Display on
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:	MOV	LCD_DATA,#LCD_CMD_BLINK	; Display blink
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Left Shift Display
;----------------------------------------------------------------------------
LCD_LSHF:	MOV	LCD_DATA,#LCD_CMD_LSHF	; Shift display to left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Right Shift Display
;----------------------------------------------------------------------------
LCD_RSHF:	MOV	LCD_DATA,#LCD_CMD_RSHF	; Shift display to right
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:          LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR     LCD_RS		; Clear LCD_RS pin
		MOV     P0,LCD_ADDR	; Move LCD_ADDR to P0
		ORL	P0,#10000000B	; Set MSB of P0
		ACALL   LCD_CLK		; Pulse LCD clock
		RET			; Return

;----------------------------------------------------------------------------
; LCD Clear Display
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCMD_LCD:	CLR	LCD_RS		; Clear LCD_RS Pin
		MOV	P0,LCD_DATA		; Write command to LCD
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET			; Return

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:	SETB	LCD_RS		; Set LCD_RS Pin
		MOV	P0,LCD_DATA	; Move LCD_DATA to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET			; Return

;----------------------------------------------------------------------------
; Put string from ROM (string must ended with 000H)
; I/P:          DPTR : Locate ROM Address
;----------------------------------------------------------------------------
STRING_LCD:	PUSH	ACC		; Put ACC. into stack
STRING_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		CJNE	A,#0,STRING_LCD_2	; Do until null terminate detect
		POP	ACC		; Pop ACC from stack
		RET			; Return

STRING_LCD_2:	MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		AJMP	STRING_LCD_1

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:	MOV	R0,#0		; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		INC	R0		; Increase loop counter
		CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
		RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:	SETB	LCD_EN		; Pulse Clock to LCD_EN
		ACALL	LCD_DELAY
		CLR	LCD_EN
		ACALL	LCD_DELAY
		RET

;----------------------------------------------------------------------------
; Dummy Delay time ONEWIRE_DELAY, LCD_DELAY, 50u, 100u, 1m, 10m, 100m, 1s
;----------------------------------------------------------------------------
ONEWIRE_DELAY:	MOV	R6,#012H	; Each loop = 75 us
ONEWIRE_DELAY_1: NOP
		NOP
		DJNZ	R6,ONEWIRE_DELAY_1
		RET

LCD_DELAY:	MOV	R7,#002		; Do 2 times
LCD_DELAY_1:	MOV	R6,#0E6H	; Each loop = 1 ms
LCD_DELAY_2:	NOP
		NOP
		DJNZ	R6,LCD_DELAY_2
		DJNZ	R7,LCD_DELAY_1
		RET

DELAY_50us:	MOV	R6,#00CH	; Each loop = 50 us
DELAY_50us_1:	NOP
		NOP
		DJNZ	R6,DELAY_50us_1
		RET

DELAY_100us:	MOV	R6,#017H	; Each loop = 100 us
DELAY_100us_1:	NOP
		NOP
		DJNZ	R6,DELAY_100us_1
		RET

DELAY_1ms:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_1ms_1:	NOP
		NOP
		DJNZ	R6,DELAY_1ms_1
		RET

DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_100ms:	MOV	R7,#100		; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

DELAY_1s:	MOV	R5,#100		; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

DELAY_750us:	MOV	R6,#229		; Each loop = 750us
DELAY_750us_2:	NOP
		DJNZ	R6,DELAY_750us_2
		RET			; Return

;----------------------------------------------------------------------------
;Define Constant < Store in Flash >
;----------------------------------------------------------------------------
NULL			EQU		0
;			0123456789ABCDEF
TITLE:		DB	" DS18S20 Hi-Res.",NULL
STRING:		DB	"Status:         ",NULL
CRC_ERR_MSG:	DB	"CRC Fail",NULL
INIT_MSG:	DB	"Initail ",NULL
UNIT_MSG:	DB	0DFH,"C ",NULL

;----------------------------------------------------------------------------

END