;****************************************************************************
; Program       : 1-Wire ROM ID 05
; Description	: 1-Wire device (also i-Button) read ROM ID 
;		: work both External power and Parasite power
; For		: MCS-51 Microcontroller Training System
; Filename	: lab2201.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************
;
;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
ONEWIRE		BIT	P2.7		; 1-Wire port

LCD_EN		BIT	P3.6		; LCD module enable (active high : level)
LCD_RS		BIT	P3.7		; LCD module register select

;----------------------------------------------------------------------------
; Define flag
;----------------------------------------------------------------------------
FLAG		EQU	02FH		; Alias FLAG @02FH
BUSY		BIT	FLAG.0		; Busy flag
ONEWIRE_ERROR	BIT	FLAG.1		; ONEWIRE error flag

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
ONEWIRE_DATA	EQU	030H		; 1-Wire data
ROM_ID		EQU	031H		; ROM ID. 8 byte (64 bit) 031H-039H
CRC_RECV	EQU	03AH		; CRC read
CRC		EQU	03BH		; CRC calculate from data
LCD_ADDR	EQU	03CH		; For keep LCD address
LCD_DATA	EQU	03DH		; For keep LCD data

ONEWIRE_LOOP	EQU	03EH		; Alias 1-Wire loop counter @03FH
ONEWIRE_CNT	EQU	03FH		; Alias 1-Wire counter @040H

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
INITIAL:	ACALL	INIT_LCD	; Call initial LCD subroutine

		MOV	LCD_ADDR,#0	; Set LCD at 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE	; Write title
		ACALL	STRING_LCD	;

MAIN:		ACALL	OW_ROM_RD	; Read ONEWIRE ID.

		JNB	ONEWIRE_ERROR,DEVICE_OK
NO_DEVICE:
		MOV	LCD_ADDR,#040H	; Set LCD at 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#NO_DEVICE_MSG	; Write title
		ACALL	STRING_LCD	;
		ACALL	DELAY_100ms
		AJMP	MAIN
		
DEVICE_OK:	MOV	R0,#ROM_ID	; Calculate CRC
		MOV	R2,#7		;
		MOV	CRC,#0		; Initial CRC=0
BUFFER_READ:	MOV     A,@R0		; Calculate each byte
		ACALL	CRC8		; Into CRC
		INC	R0		;
		DJNZ	R2,BUFFER_READ	; Do until 7 times

		MOV	CRC_RECV,@R0	; Get CRC read value

		MOV	A,CRC		; Compare with receive CRC
		CJNE	A,CRC_RECV,ERROR_CRC ; Error if not equal

		MOV	LCD_ADDR,#040H	; Show temp. start @040H
		ACALL	SET_ADDR_LCD	;
 		MOV	R0,#ROM_ID+7	; Calculate CRC
		MOV	R2,#8		;
BUFFER_SHOW:	MOV     A,@R0		; Calculate each byte
		MOV	LCD_DATA,A	; Show 2 digits HEX number
		ACALL	HEX2LCD		;
		DEC	R0		;
		DJNZ	R2,BUFFER_SHOW	; Do until 8 times
		AJMP	MAIN		; Jump to main

ERROR_CRC:	MOV	LCD_ADDR,#040H	; Show error message start @040H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#CRC_ERR_MSG
		ACALL	STRING_LCD	;
		AJMP	MAIN		; Jump to main

;----------------------------------------------------------------------------
; HEX Code to show LCD 00-FF
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
HEX2LCD:	PUSH	ACC		; Push ACC. To Stack

		SETB	LCD_RS		; Set LCD_RS Pin
		MOV	A,LCD_DATA	; Move LCD_DATA to ACC
		ANL	A,#0F0H		; Get higher 4 bit
		SWAP	A
		ADD	A,#030H		; Convert to ASCII 30H-3FH
		CJNE	A,#03AH,HEX2LCD_1 ; Check over than 039H?
HEX2LCD_1:
		JC	HEX2LCD_2
		ADD	A,#7		; Add more 7 to 'A' - 'F'
HEX2LCD_2:
		MOV	P0,A
		ACALL	LCD_CLK		; Pulse LCD Clock

		MOV	A,LCD_DATA	; Move LCD_DATA to ACC
		ANL	A,#00FH		; Get lower 4 bit
		ADD	A,#030H		; Convert to ASCII 30H-3FH
		CJNE	A,#03AH,HEX2LCD_3 ; Check over than 039H?
HEX2LCD_3:
		JC	HEX2LCD_4
		ADD	A,#7		; Add more 7 to 'A' - 'F'
HEX2LCD_4:
		MOV	P0,A
		ACALL	LCD_CLK		; Pulse LCD Clock

		POP	ACC		; Pop ACC. from Stack
		RET			; Return

;----------------------------------------------------------------------------
; CRC-8 CCITT CRC 8 calculate
;----------------------------------------------------------------------------
CRC8:		PUSH	ACC		; Push ACC. into stack
		PUSH	B		; Push B into stack
		PUSH	ACC		; Push ACC. into stack
		MOV	B,#8		; Set loop 8 times
CRC8_LOOP:	XRL	A,CRC		; Exclusive OR CRC data with data
		RRC	A		; Get bit
		MOV	A,CRC		;
		JNC	CRC_BIT_ZERO	; Check bit
		XRL	A,#18H		; Exclusive OR with CRC-8 expression
CRC_BIT_ZERO:	RRC	A		; Get bit
		MOV	CRC,A		; Replace CRC
		POP	ACC		; Pop ACC. from stach
		RR	A		; Rotate to next bit
		PUSH	ACC		; Push ACC. into stack
		DJNZ	B,CRC8_LOOP	; Do loop
		POP	ACC		; Pop ACC. from stack
		POP	B		; Pop B from stack
		POP	ACC		; Pop ACC. from stack
		RET

;----------------------------------------------------------------------------
; DS1820 Temperature Read
;----------------------------------------------------------------------------
OW_ROM_RD:
		CLR	ONEWIRE_ERROR	; Clear DS1820 error flag

		ACALL	OW_RST	; DS1820 Reset
		ACALL	OW_PRES	; DS1820 Presence
		MOV	ONEWIRE_DATA,#033H ; Write Read ROM Command
		ACALL	OW_WR	;

		MOV	R0,#ROM_ID	; Point to ROM_ID
		MOV	ONEWIRE_LOOP,#8	; Read 8 byte
OW_TEMP_LOOP:
		ACALL	OW_RD		; Read ONEWIRE
		MOV	@R0,ONEWIRE_DATA; Get byte
		INC	R0		;
		DJNZ	ONEWIRE_LOOP,OW_TEMP_LOOP	;

OW_TEMP_CLR:
		ACALL	OW_RST	; DS1820 Reset
		ACALL	OW_PRES	; DS1820 Presence

OW_TEMP_EXIT:
		RET

;----------------------------------------------------------------------------
; DS1820 Data Read
;----------------------------------------------------------------------------
OW_RD:		MOV	ONEWIRE_CNT,#8	; Set loop 8 times
		CLR	A		; Clear ACC.
OW_RD_LOOP:	CLR	ONEWIRE		; Clear ONEWIRE
		NOP			; Delay
		SETB	ONEWIRE		; Set ONEWIRE
		NOP			; Delay
		NOP			;
		MOV	C,ONEWIRE	; Get ONEWIRE to Carry Flag
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		RRC	A		;
		SETB	ONEWIRE		; Set ONEWIRE
		DJNZ	ONEWIRE_CNT,OW_RD_LOOP ; Do until 8 times
		MOV	ONEWIRE_DATA,A	; Move ACC. to ONEWIRE_DATA
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Data Write
;----------------------------------------------------------------------------
OW_WR:		MOV	ONEWIRE_CNT,#8	; Set loop 8 times
		MOV	A,ONEWIRE_DATA	; Get ONEWIRE_DATA
OW_WR_LOOP:	RRC	A		; Rotate ACC. with Carry Flag
		JNC	OW_WR_L	; Carry Flag was set ?
		CLR	ONEWIRE		; Set => TX high
		NOP			; Delay
		NOP			;
		SETB	ONEWIRE		; Set ONEWIRE
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		AJMP	OW_WR_NX	; Jump to next write
OW_WR_L:	CLR	ONEWIRE		; Clear => TX Low
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		SETB	ONEWIRE		; Set ONEWIRE
		NOP			; Delay
		NOP			;
OW_WR_NX:	DJNZ	ONEWIRE_CNT,OW_WR_LOOP ; Do until 8 times
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Reset
;----------------------------------------------------------------------------
OW_RST:		CLR	ONEWIRE		; Clear ONEWIRE
		ACALL	DELAY_750us	; Delay
		SETB	ONEWIRE		; Set ONEWIRE
		MOV	ONEWIRE_CNT,#11	; Delay
		DJNZ	ONEWIRE_CNT,$	;
		RET			; Return

;----------------------------------------------------------------------------
; DS1820 Receive Presence Pulse
;----------------------------------------------------------------------------
OW_PRES:	MOV	ONEWIRE_CNT,#110	; Set Loop wait 1
OW_PRES_1:	JNB	ONEWIRE,OW_PRES_2	; Check ONEWIRE was clear?
		DJNZ	ONEWIRE_CNT,OW_PRES_1	; Wait loop check 1
		SETB	ONEWIRE_ERROR	; Error if no response
		RET			; Return

OW_PRES_2:	MOV	ONEWIRE_CNT,#110 ; Set Loop wait 2
OW_PRES_3:	JB	ONEWIRE,OW_PRES_4	; Check ONEWIRE was clear?
		DJNZ	ONEWIRE_CNT,OW_PRES_3	; Wait loop check 1
		SETB	ONEWIRE_ERROR	; Error if no response
OW_PRES_4:	SETB	ONEWIRE		; Set ONEWIRE
		ACALL	ONEWIRE_DELAY	; Delay 75 us
		RET			; Return

;----------------------------------------------------------------------------
; Define LCD's command constant
;----------------------------------------------------------------------------
LCD_CMD_CLR	EQU	00000001B	; Clear
LCD_CMD_HOME	EQU	00000010B	; Cursor home
LCD_CMD_AUTOINC	EQU	00000110B	; Auto increment
LCD_CMD_OFF	EQU	00001000B	; Off
LCD_CMD_ON	EQU	00001100B	; On
LCD_CMD_BLINK	EQU	00001111B	; Cursor on & blink
LCD_CMD_LSHF	EQU	00011000B	; Shift display to left
LCD_CMD_RSHF	EQU	00011100B	; Shift display to right


;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:	ACALL   DELAY_100ms	; Delay

		CLR	LCD_RS		; Clear LCD_RS pin
		MOV	P0,#00111000B	; Force 8bit mode on first times
		ACALL	LCD_CLK		; Pulse LCD clock
		ACALL	DELAY_10ms	; Delay

	; force previous possible mode (4 or 8 bit) to be 8 bit
		MOV	P0,#00111000B	; Force 8bit mode again
		ACALL	LCD_CLK		; Pulse LCD clock

		ACALL	LCD_OFF		; Display off
		ACALL	LCD_AUTOINC	; Set auto increment cursor after write
		ACALL	LCD_HOME	; Return cursor to home
		ACALL	LCD_CLR		; Display cleared
		ACALL	LCD_ON		; Display on
		RET

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:	MOV	LCD_DATA,#LCD_CMD_CLR	; Display cleared
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:	MOV	LCD_DATA,#LCD_CMD_HOME	; Return cursor to uppper left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_AUTOINC:	MOV	LCD_DATA,#LCD_CMD_AUTOINC; Cursor auto increment after write
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:	MOV	LCD_DATA,#LCD_CMD_OFF	; Display off
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:		MOV	LCD_DATA,#LCD_CMD_ON	; Display on
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:	MOV	LCD_DATA,#LCD_CMD_BLINK	; Display blink
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Left Shift Display
;----------------------------------------------------------------------------
LCD_LSHF:	MOV	LCD_DATA,#LCD_CMD_LSHF	; Shift display to left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Right Shift Display
;----------------------------------------------------------------------------
LCD_RSHF:	MOV	LCD_DATA,#LCD_CMD_RSHF	; Shift display to right
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:          LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR     LCD_RS		; Clear LCD_RS pin
		MOV     P0,LCD_ADDR	; Move LCD_ADDR to P0
		ORL	P0,#10000000B	; Set MSB of P0
		ACALL   LCD_CLK		; Pulse LCD clock
		RET			; Return

;----------------------------------------------------------------------------
; LCD Clear Display
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCMD_LCD:	CLR	LCD_RS		; Clear LCD_RS Pin
		MOV	P0,LCD_DATA		; Write command to LCD
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET			; Return

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:	SETB	LCD_RS		; Set LCD_RS Pin
		MOV	P0,LCD_DATA	; Move LCD_DATA to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET			; Return

;----------------------------------------------------------------------------
; Put string from ROM (string must ended with 000H)
; I/P:          DPTR : Locate ROM Address
;----------------------------------------------------------------------------
STRING_LCD:	PUSH	ACC		; Put ACC. into stack
STRING_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		CJNE	A,#0,STRING_LCD_2	; Do until null terminate detect
		POP	ACC		; Pop ACC from stack
		RET			; Return

STRING_LCD_2:	MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		AJMP	STRING_LCD_1

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:	MOV	R0,#0		; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		INC	R0		; Increase loop counter
		CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
		RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:	SETB	LCD_EN		; Pulse Clock to LCD_EN
		ACALL	LCD_DELAY
		CLR	LCD_EN
		ACALL	LCD_DELAY
		RET

;----------------------------------------------------------------------------
; Dummy Delay time ONEWIRE_DELAY, LCD_DELAY, 50u, 100u, 1m, 10m, 100m, 1s
;----------------------------------------------------------------------------
ONEWIRE_DELAY:	MOV	R6,#012H	; Each loop = 75 us
ONEWIRE_DELAY_1: NOP
		NOP
		DJNZ	R6,ONEWIRE_DELAY_1
		RET

LCD_DELAY:	MOV	R7,#002		; Do 2 times
LCD_DELAY_1:	MOV	R6,#0E6H	; Each loop = 1 ms
LCD_DELAY_2:	NOP
		NOP
		DJNZ	R6,LCD_DELAY_2
		DJNZ	R7,LCD_DELAY_1
		RET

DELAY_50us:	MOV	R6,#00CH	; Each loop = 50 us
DELAY_50us_1:	NOP
		NOP
		DJNZ	R6,DELAY_50us_1
		RET

DELAY_100us:	MOV	R6,#017H	; Each loop = 100 us
DELAY_100us_1:	NOP
		NOP
		DJNZ	R6,DELAY_100us_1
		RET

DELAY_1ms:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_1ms_1:	NOP
		NOP
		DJNZ	R6,DELAY_1ms_1
		RET

DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_100ms:	MOV	R7,#100		; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

DELAY_1s:	MOV	R5,#100		; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

DELAY_750us:	MOV	R6,#229		; Each loop = 750us
DELAY_750us_2:	NOP
		DJNZ	R6,DELAY_750us_2
		RET			; Return

;----------------------------------------------------------------------------
;Define Constant < Store in Flash >
;----------------------------------------------------------------------------
NULL			EQU		0
;			 0123456789ABCDEF
TITLE:		DB	" 1-Wire ROM ID. ",NULL
NO_DEVICE_MSG:	DB	"   No device    ",NULL
CRC_ERR_MSG:	DB	"   CRC Failed   ",NULL

;----------------------------------------------------------------------------

END