;****************************************************************************
; Program	: Counter 01
; Description	: Frequency Counter
;		: Digit Display 0-255
; For		: MCS-51 Microcontroller Training System
; Filename	: lab2501.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
T0_PIN		BIT	P3.4	; Timer/Counter 0 PIN
LCD_EN		BIT	P3.6	; LCD Module Enable (Active High : Level)
LCD_RS		BIT	P3.7	; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG		EQU	02FH	; User FLAG
OVER		BIT	FLAG.0	; Define OVER as bit

LCD_ADDR	EQU	030H	; For keep LCD Address
LCD_DATA	EQU	031H	; For keep LCD Data

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		AJMP	INITIAL		; Jump to initial

		ORG	0003H		; IE0 Vector
		RETI			; Retrun interrupt

		ORG	000BH		; TF0 Vector
		SETB	OVER		; Set bit OVER Range
		RETI			; Retrun interrupt

		ORG	0013H		; IE1 Vector
		RETI			; Retrun interrupt

		ORG	001BH		; TF1 Vector
		RETI			; Return interrupt

		ORG	0023H		; TI+RI Vector
		RETI			; Return interrupt

		ORG	002BH		; TF2+EXF2 Vector
		RETI			; Return interrupt

INITIAL:	MOV	P0,#00000000B	; Clear Databus
		MOV	P3,#00011111B	; Clear status LCD, P3.2-3.4

MAIN:		ACALL	INIT_LCD	; Call LCD Initial subroutine

		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-0FH(Increase automatic)

MAIN_LOOP:	MOV	LCD_ADDR,#040H	; Set Address 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_RUN	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 40H-4FH(Increase automatic)

		MOV	TMOD,#00100110B ; T1 8Bit Auto, C0 8Bit Gate disable

		SETB	EA

FREQ_LOOP:	MOV	TH0,#000H	; Clear Loop Counter
		MOV	TL0,#000H	;

		CLR	OVER		; Clear Over Flag
		SETB	ET0		; Enable Counter 0 INT.
		SETB	TR0		; Start Counter 0

GATE_EN:	MOV	R7,#100		; Set gate time = 1s
GATE_EN_1:	MOV	R6,#10		; Each loop = 10 ms
GATE_EN_2:	MOV	R5,#0E6H	;
GATE_EN_3:	NOP
		NOP
		DJNZ	R5,GATE_EN_3	;
		DJNZ	R6,GATE_EN_2	;
		DJNZ	R7,GATE_EN_1	;
		CLR	TR0		; Stop Counter 0
		CLR	ET0		; Disable Counter 0 INT.

		JB	OVER,WRITE_OVER ; Check Over Range ?

		MOV	LCD_ADDR,#048H	; Set Address 48H
		ACALL	SET_ADDR_LCD	;
		MOV	LCD_DATA,TL0	; Get Counter Value
		ACALL	BYTE2LCD		; Write to LCD

		ACALL	DELAY_250ms	; Delay Screen
		AJMP	FREQ_LOOP	; Jump to Read Frequency again

WRITE_OVER:	MOV	LCD_ADDR,#040H	; Set Address 40H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_OVER	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 40H-4FH(Increase automatic)
		ACALL	DELAY_250ms	; Delay Screen

		AJMP	MAIN_LOOP	; Jump to Main loop

;----------------------------------------------------------------------------
; Show byte value as 3 digit decimal
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
BYTE2LCD:	PUSH	ACC		; Push ACC.
		MOV	A,LCD_DATA	; Get Data
		MOV	B,#100		;
		DIV	AB		; Divide by 100
		ADD	A,#030H		; Convert to ASCII
		CJNE	A,#030H,BYTE2LCD_1 ; Check x100 = 0 ?
		MOV	A,#' '		; 0 => Write Space
BYTE2LCD_1:	MOV	LCD_DATA,A	;
		ACALL	WRCHAR_LCD	; Write x100
		MOV	A,B		;
		MOV	B,#10		;
		DIV	AB		; Divide by 10
		ADD	A,#030H		; Convert to ASCII
		MOV	LCD_DATA,A	;
		ACALL	WRCHAR_LCD	; Write Lower HEX Code
		MOV	A,B		; Get Remainder x1
		ADD	A,#030H		; Convert to ASCII
		MOV	LCD_DATA,A	;
		ACALL	WRCHAR_LCD	; Write Lower HEX Code
		POP	ACC		; Pop ACC.
		RET			; Return

;----------------------------------------------------------------------------
; Define LCD's command constant
;----------------------------------------------------------------------------
LCD_CMD_CLR	EQU	00000001B	; Clear
LCD_CMD_HOME	EQU	00000010B	; Cursor home
LCD_CMD_AUTOINC	EQU	00000110B	; Auto increment
LCD_CMD_OFF	EQU	00001000B	; Off
LCD_CMD_ON	EQU	00001100B	; On
LCD_CMD_BLINK	EQU	00001111B	; Cursor on & blink
LCD_CMD_LSHF	EQU	00011000B	; Shift display to left
LCD_CMD_RSHF	EQU	00011100B	; Shift display to right


;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:	ACALL   DELAY_100ms	; Delay

		CLR	LCD_RS		; Clear LCD_RS pin
		MOV	P0,#00111000B	; Force 8bit mode on first times
		ACALL	LCD_CLK		; Pulse LCD clock
		ACALL	DELAY_10ms	; Delay

	; force previous possible mode (4 or 8 bit) to be 8 bit
		MOV	P0,#00111000B	; Force 8bit mode again
		ACALL	LCD_CLK		; Pulse LCD clock

		ACALL	LCD_OFF		; Display off
		ACALL	LCD_AUTOINC	; Set auto increment cursor after write
		ACALL	LCD_HOME	; Return cursor to home
		ACALL	LCD_CLR		; Display cleared
		ACALL	LCD_ON		; Display on
		RET

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:	MOV	LCD_DATA,#LCD_CMD_CLR	; Display cleared
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:	MOV	LCD_DATA,#LCD_CMD_HOME	; Return cursor to uppper left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_AUTOINC:	MOV	LCD_DATA,#LCD_CMD_AUTOINC; Cursor auto increment after write
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:	MOV	LCD_DATA,#LCD_CMD_OFF	; Display off
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:		MOV	LCD_DATA,#LCD_CMD_ON	; Display on
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:	MOV	LCD_DATA,#LCD_CMD_BLINK	; Display blink
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Left Shift Display
;----------------------------------------------------------------------------
LCD_LSHF:	MOV	LCD_DATA,#LCD_CMD_LSHF	; Shift display to left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Right Shift Display
;----------------------------------------------------------------------------
LCD_RSHF:	MOV	LCD_DATA,#LCD_CMD_RSHF	; Shift display to right
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:          LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR     LCD_RS		; Clear LCD_RS pin
		MOV     P0,LCD_ADDR	; Move LCD_ADDR to P0
		ORL	P0,#10000000B	; Set MSB of P0
		ACALL   LCD_CLK		; Pulse LCD clock
		RET			; Return

;----------------------------------------------------------------------------
; LCD Clear Display
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCMD_LCD:	CLR	LCD_RS		; Clear LCD_RS Pin
		MOV	P0,LCD_DATA		; Write command to LCD
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET			; Return

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:	SETB	LCD_RS		; Set LCD_RS Pin
		MOV	P0,LCD_DATA	; Move LCD_DATA to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET						; Return

;----------------------------------------------------------------------------
; Put string from ROM (string must ended with 000H)
; I/P:          DPTR : Locate ROM Address
;----------------------------------------------------------------------------
STRING_LCD:	PUSH	ACC		; Put ACC. into stack
STRING_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		CJNE	A,#0,STRING_LCD_2	; Do until null terminate detect
		POP	ACC		; Pop ACC from stack
		RET			; Return

STRING_LCD_2:	MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		AJMP	STRING_LCD_1

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:	MOV	R0,#0		; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		INC	R0		; Increase loop counter
		CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
		RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:	SETB	LCD_EN		; Pulse Clock to LCD_EN
		ACALL	LCD_DELAY
		CLR	LCD_EN
		ACALL	LCD_DELAY
		RET

;----------------------------------------------------------------------------
; Dummy Delay time LCD_DELAY, 10m, 100m, 250m, 1s
;----------------------------------------------------------------------------
LCD_DELAY:	MOV	R7,#002		; Do 2 times
LCD_DELAY_1:	MOV	R6,#0E6H	; Each loop = 1 ms
LCD_DELAY_2:	NOP
		NOP
		DJNZ	R6,LCD_DELAY_2
		DJNZ	R7,LCD_DELAY_1
		RET

DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_100ms:	MOV	R7,#100		; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

DELAY_250ms:	MOV	R7,#250		; Do 250 times
DELAY_250ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_250ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_250ms_2
		DJNZ	R7,DELAY_250ms_1
		RET

DELAY_1s:	MOV	R5,#100		; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;			 0123456789ABCDEF
TITLE_1:	DB	' Freq. Counter  '

SCR_RUN:	DB	'FREQ :        Hz'
SCR_OVER:	DB	'Input over range'

		END