;****************************************************************************
; Program	: Serial UART 02
; Description	: Serial UART Exchange Data with PC
;		: using Bit polled check method
; For		: MCS-51 Microcontroller Training System
; Filename	: lab2602.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
BUFFER		EQU		030H	; Buffer

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
MAIN:		MOV	TMOD,#021H	; T1 8Bit Auto, T0 16Bit
		MOV	TH1,#0FDH	; 9600 bps Timer1 Default
		MOV	TL1,#0FDH	;

		SETB	TR1		; Start Timer1

		MOV	SCON,#040H	; Mode1 RX Disable

		MOV	DPTR,#SERIAL_TEXT ; Set Pointer Serial TX
		ACALL	TX_TEXT		; TX Text to Serial Port

		MOV	DPTR,#HELLO_TEXT ; Set Pointer Serial TX
		ACALL	TX_TEXT		; TX Text to Serial Port

START_LOOP:	MOV	R0,#BUFFER	; Clear BUFFER to start pointer
		SETB	REN		; Set RX Enable

LOOP:		JNB	RI,$		; Wait byte received
		CLR	RI		;
		MOV	A,SBUF		; Get byte from SBUF
		MOV	@R0,A		;
		CJNE	A,#00DH,RX_NEXT ; Check Enter key receive?

		MOV	A,R0
		CJNE	A,#BUFFER,DATA_VALID ; Check enter key only?

DATA_EMPTY:	MOV	DPTR,#NODATA_TEXT1	; Set Pointer Serial TX
		ACALL	TX_TEXT		; TX Text to Serial Port
		AJMP	START_LOOP

DATA_VALID:	MOV	DPTR,#GOT_TEXT1	; Set Pointer Serial TX
		ACALL	TX_TEXT		; TX Text to Serial Port

		MOV	A,R0		; Keep end of message pointer

		MOV	R0,#BUFFER	; Reset Pointer
REPLY_LOOP:	CLR	TI		; Send reply byte to PC
		MOV	SBUF,@R0	; by get data from BUFFER
		JNB	TI,$		;
		CLR	TI		;

		INC	R0		; Increase to next pointer
		CJNE	A,0,REPLY_LOOP	; Check end of message already?

		MOV	DPTR,#GOT_TEXT2	; Set Pointer Serial TX
		ACALL	TX_TEXT		; TX Text to Serial Port
		AJMP	START_LOOP

RX_NEXT:	INC	R0              ; Increase pointer
		AJMP	LOOP		; Back to do loop

;----------------------------------------------------------------------------
; TX Serial Text from ROM Pointer
;----------------------------------------------------------------------------
TX_TEXT:	CLR	TI		; Clear TI
TX_LOOP:	CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Get Data from ROM with Pointer
		INC	DPTR		; Increase Pointer
		CJNE	A,#0,TX_CHAR	; Check null terminated 000H Char.
		RET			; End => Return

TX_CHAR:	MOV	SBUF,A		; Send Data to SBUF
		JNB	TI,$		; Wait until TX already (TI=1)
		CLR	TI		; Clear TI
		AJMP	TX_LOOP		; Jump to TX_LOOP

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
NULL		EQU	0

SERIAL_TEXT:	DB	00AH,00DH
		DB	00AH,00DH
		DB	'Serial UART Communication < baudrate 9600bps > ',00AH,00DH
		DB	'for MCS-51 Microcontroller Training System',00AH,00DH
		DB	'Copyright (C) 2000 Innovative Experiment Co.,Ltd.',00AH,00DH
		DB	00AH,00DH,NULL

HELLO_TEXT:	DB	'Hello.. PC. I',027H,'m waiting message from you.',00AH,00DH
		DB	'type any message then press ENTER key.',00AH,00DH,NULL

NODATA_TEXT1:	DB	'No more data received',00AH,00DH,NULL

GOT_TEXT1:	DB	'I',027H,'d got message [',NULL
GOT_TEXT2:	DB	'] from you.',00AH,00DH,NULL

		END